@@ -20,11 +20,11 @@
 
 import logging
 import copy
-from functools import reduce  # pylint: disable=redefined-builtin
+from functools import reduce
 import time
 import types
 import traceback
-import subprocess
+import subprocess  # nosec - internal
 from collections import OrderedDict
 from nose.tools import nottest
 from lava_common.timeout import Timeout
@@ -32,13 +32,14 @@ from lava_common.exceptions import (
     LAVABug,
     LAVAError,
     JobError,
+    TestError,
+    LAVATimeoutError,
 )
 from lava_dispatcher.log import YAMLLogger
-from lava_dispatcher.utils.lxc import is_lxc_requested
 from lava_dispatcher.utils.strings import seconds_to_str
 
 
-class InternalObject(object):  # pylint: disable=too-few-public-methods
+class InternalObject:  # pylint: disable=too-few-public-methods
     """
     An object within the dispatcher pipeline which should not be included in
     the description of the pipeline.
@@ -46,7 +47,7 @@ class InternalObject(object):  # pylint: disable=too-few-public-methods
     pass
 
 
-class Pipeline(object):  # pylint: disable=too-many-instance-attributes
+class Pipeline:  # pylint: disable=too-many-instance-attributes
     """
     Pipelines ensure that actions are run in the correct sequence whilst
     allowing for retries and other requirements.
@@ -126,7 +127,7 @@ class Pipeline(object):  # pylint: disable=too-many-instance-attributes
         # pylint: disable=protected-access
         action._override_action_timeout(dict_merge_get(timeouts, 'action'))
         action._override_action_timeout(subdict_merge_get(timeouts, 'actions', action.name))
-        action._override_action_timeout(parameters.get('timeout', None))
+        action._override_action_timeout(parameters.get('timeout'))
 
         action._override_connection_timeout(dict_merge_get(timeouts, 'connection'))
         action._override_connection_timeout(subdict_merge_get(timeouts, 'connections', action.name))
@@ -202,13 +203,14 @@ class Pipeline(object):  # pylint: disable=too-many-instance-attributes
         # Diagnosis is not allowed to alter the connection, do not use the return value.
         return None
 
-    def run_actions(self, connection, max_end_time, args=None):  # pylint: disable=too-many-branches,too-many-statements,too-many-locals
+    def run_actions(self, connection, max_end_time):  # pylint: disable=too-many-branches,too-many-statements,too-many-locals
         for action in self.actions:
             failed = False
             namespace = action.parameters.get('namespace', 'common')
             # Begin the action
             try:
-                with action.timeout(max_end_time) as action_max_end_time:
+                parent = self.parent if self.parent else self.job
+                with action.timeout(parent, max_end_time) as action_max_end_time:
                     # Add action start timestamp to the log message
                     # Log in INFO for root actions and in DEBUG for the other actions
                     timeout = seconds_to_str(action_max_end_time - action.timeout.start)
@@ -219,8 +221,21 @@ class Pipeline(object):  # pylint: disable=too-many-instance-attributes
                     else:
                         action.logger.debug(msg)
 
-                    new_connection = action.run(connection,
-                                                action_max_end_time, args)
+                    new_connection = action.run(connection, action_max_end_time)
+            except LAVATimeoutError as exc:
+                action.logger.exception(str(exc))
+                # allows retries without setting errors, which make the job incomplete.
+                failed = True
+                action.results = {'fail': str(exc)}
+                if action.timeout.can_skip(action.parameters):
+                    if self.parent is None:
+                        action.logger.warning(
+                            "skip_timeout is set for %s - continuing to next action block." % (action.name))
+                    else:
+                        raise
+                    new_connection = None
+                else:
+                    raise TestError(str(exc))
             except LAVAError as exc:
                 action.logger.exception(str(exc))
                 # allows retries without setting errors, which make the job incomplete.
@@ -253,7 +268,7 @@ class Pipeline(object):  # pylint: disable=too-many-instance-attributes
         return connection
 
 
-class Action(object):  # pylint: disable=too-many-instance-attributes,too-many-public-methods
+class Action:  # pylint: disable=too-many-instance-attributes,too-many-public-methods
 
     def __init__(self):
         """
@@ -283,7 +298,6 @@ class Action(object):  # pylint: disable=too-many-instance-attributes,too-many-p
         self.max_retries = 1  # unless the strategy or the job parameters change this, do not retry
         self.diagnostics = []
         self.protocols = []  # list of protocol objects supported by this action, full list in job.protocols
-        self.lxc_cmd_prefix = []
         self.connection_timeout = Timeout(self.name, exception=self.timeout_exception)
         self.character_delay = 0
         self.force_prompt = False
@@ -405,6 +419,13 @@ class Action(object):  # pylint: disable=too-many-instance-attributes,too-many-p
         except ValueError:
             raise LAVABug("Action results need to be a dictionary")
 
+    def get_constant(self, key, prefix):
+        # whilst deployment data is still supported, check if the key exists there.
+        # once deployment_data is removed, merge with device.get_constant
+        if self.parameters.get('deployment_data'):
+            return self.parameters['deployment_data'][key]
+        return self.job.device.get_constant(key, prefix=prefix)
+
     def validate(self):
         """
         This method needs to validate the parameters to the action. For each
@@ -431,11 +452,6 @@ class Action(object):  # pylint: disable=too-many-instance-attributes,too-many-p
         if not self.section:
             self.errors = "action %s (%s) has no section set" % (self.name, self)
 
-        # Decide whether we need lxc_cmd_prefix or not
-        lxc_name = is_lxc_requested(self.job)
-        if lxc_name:
-            self.lxc_cmd_prefix = ['lxc-attach', '-n', lxc_name, '--']
-
         # Collect errors from internal pipeline actions
         if self.internal_pipeline:
             self.internal_pipeline.validate_actions()
@@ -477,9 +493,9 @@ class Action(object):  # pylint: disable=too-many-instance-attributes,too-many-p
         command_list = ['nice'] + [str(s) for s in command_list]
         self.logger.debug("%s", ' '.join(command_list))
         try:
-            log = subprocess.check_output(command_list, stderr=subprocess.STDOUT,
+            log = subprocess.check_output(command_list, stderr=subprocess.STDOUT,  # nosec - internal
                                           cwd=cwd)
-            log = log.decode('utf-8', errors="replace")  # pylint: disable=redefined-variable-type
+            log = log.decode('utf-8', errors="replace")
         except subprocess.CalledProcessError as exc:
             # the errors property doesn't support removing errors
             errors = []
@@ -487,8 +503,8 @@ class Action(object):  # pylint: disable=too-many-instance-attributes,too-many-p
                 errors.append(exc.output.strip().decode('utf-8', errors="replace"))
             else:
                 errors.append(str(exc))
-            msg = '[%s] command %s\nmessage %s\noutput %s\n' % (
-                self.name, [i.strip() for i in exc.cmd], str(exc), str(exc).split('\n'))
+            msg = 'action: %s\ncommand: %s\nmessage: %s\noutput: %s\n' % (
+                self.name, [i.strip() for i in exc.cmd], str(exc), "\n".join(errors))
 
             # the exception is raised due to a non-zero exc.returncode
             if allow_fail:
@@ -540,7 +556,7 @@ class Action(object):  # pylint: disable=too-many-instance-attributes,too-many-p
                     self.set_namespace_data(
                         action=protocol.name, label=protocol.name, key=message[0], value=message[1])
 
-    def run(self, connection, max_end_time, args=None):
+    def run(self, connection, max_end_time):
         """
         This method is responsible for performing the operations that an action
         is supposed to do.
@@ -554,13 +570,12 @@ class Action(object):  # pylint: disable=too-many-instance-attributes,too-many-p
 
         :param connection: The Connection object to use to run the steps
         :param max_end_time: The maximum time before this action will timeout.
-        :param args: Command and arguments to run
         :raise: Classes inheriting from BaseAction must handle
         all exceptions possible from the command and re-raise
         """
         self.call_protocols()
         if self.internal_pipeline:
-            return self.internal_pipeline.run_actions(connection, max_end_time, args)
+            return self.internal_pipeline.run_actions(connection, max_end_time)
         if connection:
             connection.timeout = self.connection_timeout
         return connection
@@ -588,10 +603,10 @@ class Action(object):  # pylint: disable=too-many-instance-attributes,too-many-p
         e.g. SignalMatch
         """
         data = {}
-        attrs = set([attr for attr in dir(self)
+        attrs = set((attr for attr in dir(self)
                      if not attr.startswith('_') and getattr(self, attr) and not
                      isinstance(getattr(self, attr), types.MethodType) and not
-                     isinstance(getattr(self, attr), InternalObject)])
+                     isinstance(getattr(self, attr), InternalObject)))
 
         # noinspection PySetFunctionToLiteral
         for attr in attrs - set([
@@ -610,10 +625,10 @@ class Action(object):  # pylint: disable=too-many-instance-attributes,too-many-p
                 data['protocols'] = {}
                 for protocol in getattr(self, attr):
                     data['protocols'][protocol.name] = {}
-                    protocol_attrs = set([attr for attr in dir(protocol)
+                    protocol_attrs = set((attr for attr in dir(protocol)
                                           if not attr.startswith('_') and getattr(protocol, attr) and not
                                           isinstance(getattr(protocol, attr), types.MethodType) and not
-                                          isinstance(getattr(protocol, attr), InternalObject)])
+                                          isinstance(getattr(protocol, attr), InternalObject)))
                     for protocol_attr in protocol_attrs:
                         if protocol_attr not in ['logger']:
                             data['protocols'][protocol.name][protocol_attr] = getattr(protocol, protocol_attr)
@@ -621,9 +636,6 @@ class Action(object):  # pylint: disable=too-many-instance-attributes,too-many-p
                 data[attr] = dict(getattr(self, attr))
             else:
                 data[attr] = getattr(self, attr)
-        if 'deployment_data' in self.parameters:
-            data['parameters'] = dict()
-            data['parameters']['deployment_data'] = self.parameters['deployment_data'].__data__
         return data
 
     def get_namespace_keys(self, action, parameters=None):
@@ -651,7 +663,7 @@ class Action(object):  # pylint: disable=too-many-instance-attributes,too-many-p
         """
         params = parameters if parameters else self.parameters
         namespace = params['namespace']
-        value = self.data.get(namespace, {}).get(action, {}).get(label, {}).get(key, None)  # pylint: disable=no-member
+        value = self.data.get(namespace, {}).get(action, {}).get(label, {}).get(key)  # pylint: disable=no-member
         if value is None:
             return None
         return copy.deepcopy(value) if deepcopy else value
