@@ -94,7 +94,7 @@ class VlandProtocol(Protocol):
         try:
             self.sock.connect((self.settings['vland_hostname'], self.settings['port']))
             return True
-        except socket.error as exc:
+        except OSError as exc:
             self.logger.exception(
                 "socket error on connect: %d %s %s",
                 exc.errno, self.settings['vland_hostname'], self.settings['port'])
@@ -106,15 +106,15 @@ class VlandProtocol(Protocol):
         msg_len = len(message)
         try:
             # send the length as 32bit hexadecimal
-            ret_bytes = self.sock.send("%08X" % msg_len)
+            ret_bytes = self.sock.send(b"%08X" % msg_len)
             if ret_bytes == 0:
                 self.logger.debug("zero bytes sent for length - connection closed?")
                 return False
-            ret_bytes = self.sock.send(message)
+            ret_bytes = self.sock.send(message.encode())
             if ret_bytes == 0:
                 self.logger.debug("zero bytes sent for message - connection closed?")
                 return False
-        except socket.error as exc:
+        except OSError as exc:
             self.logger.exception("socket error '%d' on send", exc.message)
             self.sock.close()
             return False
@@ -130,9 +130,9 @@ class VlandProtocol(Protocol):
             recv_count = 0
             response = ''
             while recv_count < msg_count:
-                response += self.sock.recv(self.blocks)
+                response += self.sock.recv(self.blocks).decode()
                 recv_count += self.blocks
-        except socket.error as exc:
+        except OSError as exc:
             self.logger.exception("socket error '%d' on response", exc.errno)
             self.sock.close()
             return json.dumps({"response": "wait"})
