@@ -39,7 +39,11 @@ from lava_common.exceptions import (
 from lava_dispatcher.parser import JobParser
 from lava_dispatcher.job import Job
 from lava_dispatcher.device import NewDevice
-from lava_scheduler_app.schema import validate_device, SubmissionException
+from lava_scheduler_app.schema import (
+    validate_device,
+    validate_submission,
+    SubmissionException
+)
 from lava_dispatcher.actions.deploy.image import DeployImages
 from lava_dispatcher.test.utils import DummyLogger
 
@@ -66,7 +70,7 @@ class StdoutTestCase(unittest.TestCase):  # pylint: disable=too-many-public-meth
             with open(y_file, 'w') as describe:
                 yaml.dump(job.pipeline.describe(False), describe)
         with open(y_file, 'r') as f_ref:
-            return yaml.load(f_ref)
+            return yaml.safe_load(f_ref)
 
 
 class TestAction(StdoutTestCase):  # pylint: disable=too-many-public-methods
@@ -86,7 +90,7 @@ class TestPipelineInit(StdoutTestCase):  # pylint: disable=too-many-public-metho
             self.ran = False
             super().__init__()
 
-        def run(self, connection, max_end_time, args=None):
+        def run(self, connection, max_end_time):
             self.ran = True
 
         def post_process(self):
@@ -147,7 +151,7 @@ class TestValidation(StdoutTestCase):  # pylint: disable=too-many-public-methods
         self.assertEqual([1, 2], pipe.errors)
 
 
-class Factory(object):
+class Factory:
     """
     Not Model based, this is not a Django factory.
     Factory objects are dispatcher based classes, independent
@@ -173,9 +177,9 @@ class Factory(object):
         string_loader = jinja2.DictLoader({'%s.jinja2' % hostname: jinja_data})
         path = os.path.dirname(self.CONFIG_PATH)
         type_loader = jinja2.FileSystemLoader([os.path.join(path, 'device-types')])
-        env = jinja2.Environment(
+        env = jinja2.Environment(  # nosec - YAML, not HTML, no XSS scope.
             loader=jinja2.ChoiceLoader([string_loader, type_loader]),
-            trim_blocks=True)
+            trim_blocks=True, autoescape=False)
         return env.get_template("%s.jinja2" % hostname)
 
     def render_device_dictionary(self, hostname, data, job_ctx=None):
@@ -191,7 +195,7 @@ class Factory(object):
         """
         rendered = self.render_device_dictionary(hostname, data, job_ctx)
         try:
-            ret = validate_device(yaml.load(rendered))
+            ret = validate_device(yaml.safe_load(rendered))
         except (SubmissionException, ConfigurationError) as exc:
             print('#######')
             print(rendered)
@@ -214,10 +218,13 @@ class Factory(object):
         rendered = self.render_device_dictionary(hostname, data, job_ctx)
         return (rendered, data)
 
-    def create_custom_job(self, template, job_data):
-        job_ctx = job_data.get('context', None)
+    def create_custom_job(self, template, job_data, job_ctx=None):
+        if job_ctx:
+            job_data['context'] = job_ctx
+        else:
+            job_ctx = job_data.get('context')
         (data, device_dict) = self.create_device(template, job_ctx)
-        device = NewDevice(yaml.load(data))
+        device = NewDevice(yaml.safe_load(data))
         if self.debug:
             print('####### Device configuration #######')
             print(data)
@@ -233,21 +240,21 @@ class Factory(object):
         job.logger = DummyLogger()
         return job
 
-    def create_job(self, template, filename):
+    def create_job(self, template, filename, job_ctx=None):
         y_file = os.path.join(os.path.dirname(__file__), filename)
         with open(y_file) as sample_job_data:
-            job_data = yaml.load(sample_job_data.read())
-        return self.create_custom_job(template, job_data)
+            job_data = yaml.safe_load(sample_job_data.read())
+        return self.create_custom_job(template, job_data, job_ctx)
 
     def create_fake_qemu_job(self):
         return self.create_job('qemu01.jinja2', 'sample_jobs/basics.yaml')
 
-    def create_kvm_job(self, filename):  # pylint: disable=no-self-use
+    def create_kvm_job(self, filename, check_job=False):  # pylint: disable=no-self-use
         """
         Custom function to allow for extra exception handling.
         """
         (data, device_dict) = self.create_device('kvm01.jinja2')
-        device = NewDevice(yaml.load(data))
+        device = NewDevice(yaml.safe_load(data))
         if self.debug:
             print('####### Device configuration #######')
             print(data)
@@ -255,9 +262,15 @@ class Factory(object):
         self.validate_data('hi6220-hikey-01', device_dict)
         kvm_yaml = os.path.join(os.path.dirname(__file__), filename)
         parser = JobParser()
+        job_data = ''
+        with open(kvm_yaml) as sample_job_data:
+            job_data = yaml.safe_load(sample_job_data.read())
+        if self.debug:
+            print('########## Test Job Submission validation #######')
+        if check_job:  # FIXME: all submissions should validate.
+            validate_submission(job_data)
         try:
-            with open(kvm_yaml) as sample_job_data:
-                job = parser.parse(sample_job_data, device, 4212, None, "")
+            job = parser.parse(yaml.dump(job_data), device, 4212, None, "")
             job.logger = DummyLogger()
         except LAVAError as exc:
             print(exc)
@@ -276,7 +289,7 @@ class TestPipeline(StdoutTestCase):  # pylint: disable=too-many-public-methods
             self.ran = False
             super().__init__()
 
-        def run(self, connection, max_end_time, args=None):
+        def run(self, connection, max_end_time):
             time.sleep(1)
             self.ran = True
 
@@ -430,11 +443,11 @@ class TestPipeline(StdoutTestCase):  # pylint: disable=too-many-public-methods
         self.assertEqual(job.compatibility, DeployImages.compatibility)
         kvm_yaml = os.path.join(os.path.dirname(__file__), 'sample_jobs/kvm.yaml')
         with open(kvm_yaml, 'r') as kvm_yaml:
-            job_def = yaml.load(kvm_yaml)
+            job_def = yaml.safe_load(kvm_yaml)
         job_def['compatibility'] = job.compatibility
         parser = JobParser()
         (rendered, data) = factory.create_device('kvm01.jinja2')
-        device = yaml.load(rendered)
+        device = yaml.safe_load(rendered)
         try:
             job = parser.parse(yaml.dump(job_def), device, 4212, None, "")
         except NotImplementedError:
@@ -483,7 +496,7 @@ class TestFakeActions(StdoutTestCase):  # pylint: disable=too-many-public-method
     class KeepConnection(Action):
         name = "keep-connection"
 
-        def run(self, connection, max_end_time, args=None):
+        def run(self, connection, max_end_time):
             pass
 
         def post_process(self):
@@ -492,7 +505,7 @@ class TestFakeActions(StdoutTestCase):  # pylint: disable=too-many-public-method
     class MakeNewConnection(Action):
         name = "make-new-connection"
 
-        def run(self, connection, max_end_time, args=None):
+        def run(self, connection, max_end_time):
             new_connection = object()
             return new_connection
 
@@ -537,7 +550,7 @@ class TestStrategySelector(StdoutTestCase):
     """
     Check the lambda operation
     """
-    class Base(object):
+    class Base:
         priority = 0
 
     class First(Base):
