@@ -20,7 +20,6 @@
 
 import re
 import os
-import sys
 import glob
 import stat
 import yaml
@@ -28,10 +27,9 @@ import shutil
 import pexpect
 import tempfile
 import unittest
-import subprocess
+import subprocess  # nosec - unit test support.
 from nose.tools import nottest
 from lava_dispatcher.power import FinalizeAction
-from lava_dispatcher.device import NewDevice
 from lava_dispatcher.parser import JobParser
 from lava_common.exceptions import InfrastructureError
 from lava_dispatcher.actions.test.shell import TestShellRetry, PatternFixup
@@ -49,7 +47,10 @@ from lava_dispatcher.actions.deploy.testdef import (
 from lava_dispatcher.actions.boot import BootAction
 from lava_dispatcher.actions.deploy.overlay import OverlayAction
 from lava_dispatcher.actions.deploy.download import DownloaderAction
-from lava_dispatcher.test.utils import infrastructure_error
+from lava_dispatcher.test.utils import (
+    infrastructure_error,
+    infrastructure_error_multi_paths,
+)
 
 
 # pylint: disable=duplicate-code
@@ -128,11 +129,11 @@ class TestDefinitionHandlers(StdoutTestCase):  # pylint: disable=too-many-public
         testdef.validate()
         self.assertEqual([], testdef.errors)
         (rendered, _) = self.factory.create_device('kvm01.jinja2')
-        device = yaml.load(rendered)
+        device = yaml.safe_load(rendered)
         kvm_yaml = os.path.join(os.path.dirname(__file__), 'sample_jobs/kvm.yaml')
         parser = JobParser()
         with open(kvm_yaml, 'r') as sample_job_data:
-            content = yaml.load(sample_job_data)
+            content = yaml.safe_load(sample_job_data)
         data = [block['test'] for block in content['actions'] if 'test' in block][0]
         definitions = [block for block in data['definitions'] if 'path' in block][0]
         definitions['name'] = 'smoke tests'
@@ -174,12 +175,14 @@ class TestDefinitionHandlers(StdoutTestCase):  # pylint: disable=too-many-public
             'lava-target-mac',
             'lava-target-storage',
             'lava-test-case',
+            'lava-test-event',
             'lava-test-feedback',
             'lava-test-reference',
             'lava-test-runner',
             'lava-test-set',
             'lava-test-shell',
             'lava-test-raise',
+            'lava-common-functions'
         ]
 
         overlay = None
@@ -200,6 +203,12 @@ class TestDefinitionHandlers(StdoutTestCase):  # pylint: disable=too-many-public
         self.assertCountEqual(check_list, script_list)
         self.assertEqual(overlay.xmod, stat.S_IRWXU | stat.S_IXGRP | stat.S_IRGRP | stat.S_IXOTH | stat.S_IROTH)
 
+    def test_overlay_override(self):
+        job = self.factory.create_job('qemu01.jinja2', 'sample_jobs/kvm-context.yaml')
+        deploy = [action for action in job.pipeline.actions if action.name == 'deployimages'][0]
+        overlay = [action for action in deploy.internal_pipeline.actions if action.name == 'lava-overlay'][0]
+        self.assertEqual('/sysroot/lava-%s', overlay.get_constant('lava_test_results_dir', 'posix'))
+
 
 class TestDefinitionSimple(StdoutTestCase):  # pylint: disable=too-many-public-methods
 
@@ -378,7 +387,7 @@ def check_rpcinfo(server='127.0.0.1'):
     returns True on failure.
     """
     try:
-        subprocess.check_output(['/usr/sbin/rpcinfo', '-u', server, 'nfs', 3])
+        subprocess.check_output(['/usr/sbin/rpcinfo', '-u', server, 'nfs', '3'])  # nosec - unit test support.
     except (OSError, subprocess.CalledProcessError):
         return True
     return False
@@ -397,13 +406,13 @@ class TestDefinitions(StdoutTestCase):
         super().setUp()
         self.testdef = os.path.join(os.path.dirname(__file__), 'testdefs', 'params.yaml')
         self.res_data = os.path.join(os.path.dirname(__file__), 'testdefs', 'result-data.txt')
-        factory = UBootFactory()
-        self.job = factory.create_bbb_job("sample_jobs/bbb-nfs-url.yaml")
+        self.factory = UBootFactory()
+        self.job = self.factory.create_bbb_job("sample_jobs/bbb-nfs-url.yaml")
 
     def test_pattern(self):
         self.assertTrue(os.path.exists(self.testdef))
         with open(self.testdef, 'r') as par:
-            params = yaml.load(par)
+            params = yaml.safe_load(par)
         self.assertIn('parse', params.keys())
         line = 'test1a: pass'
         self.assertEqual(
@@ -420,20 +429,20 @@ class TestDefinitions(StdoutTestCase):
         # without a name from a testdef, the pattern is not valid.
         self.assertFalse(pattern.valid())
         with open(self.testdef, 'r') as par:
-            params = yaml.load(par)
+            params = yaml.safe_load(par)
         pattern = PatternFixup(testdef=params, count=0)
         self.assertTrue(pattern.valid())
 
-    # @unittest.skipIf(check_rpcinfo(), "rpcinfo returns non-zero for nfs")
+    @unittest.skipIf(check_rpcinfo(), "rpcinfo returns non-zero for nfs")
     def test_definition_lists(self):  # pylint: disable=too-many-locals
         self.job.validate()
         tftp_deploy = [action for action in self.job.pipeline.actions if action.name == 'tftp-deploy'][0]
         prepare = [action for action in tftp_deploy.internal_pipeline.actions if action.name == 'prepare-tftp-overlay'][0]
         overlay = [action for action in prepare.internal_pipeline.actions if action.name == 'lava-overlay'][0]
         apply_o = [action for action in prepare.internal_pipeline.actions if action.name == 'apply-overlay-tftp'][0]
-        self.assertIsNone(apply_o.parameters.get('nfs_url', None))
-        self.assertIsInstance(apply_o.parameters.get('persistent_nfs', None), dict)
-        self.assertIsInstance(apply_o.parameters['persistent_nfs'].get('address', None), str)
+        self.assertIsNone(apply_o.parameters.get('nfs_url'))
+        self.assertIsInstance(apply_o.parameters.get('persistent_nfs'), dict)
+        self.assertIsInstance(apply_o.parameters['persistent_nfs'].get('address'), str)
         definition = [action for action in overlay.internal_pipeline.actions if action.name == 'test-definition'][0]
         git_repos = [action for action in definition.internal_pipeline.actions if action.name == 'git-repo-action']
         self.assertIn('common', self.job.context)
@@ -460,7 +469,7 @@ class TestDefinitions(StdoutTestCase):
         )
         # fake up a run step
         with open(self.testdef, 'r') as par:
-            params = yaml.load(par)
+            params = yaml.safe_load(par)
         self.assertEqual(
             r'(?P<test_case_id>.*-*):\s+(?P<result>(pass|fail))',
             params['parse']['pattern'])
@@ -500,7 +509,7 @@ test3a: skip
 \"test4a:\" \"unknown\"
         """
         with open(self.testdef, 'r') as par:
-            params = yaml.load(par)
+            params = yaml.safe_load(par)
         pattern = params['parse']['pattern']
         re_pat = re.compile(pattern, re.M)
         match = re.search(re_pat, data)
@@ -513,3 +522,33 @@ test3a: skip
         self.assertEqual(child.after.encode('utf-8'), b'test2a: fail')
         child.expect([re_pat, pexpect.EOF])
         self.assertEqual(child.after, pexpect.EOF)
+
+    @unittest.skipIf(infrastructure_error_multi_paths(
+        ['lxc-info', 'img2simg', 'simg2img']),
+        "lxc or img2simg or simg2img not installed")
+    def test_deployment_data(self):
+        job = self.factory.create_job('hi960-hikey-01.jinja2', 'sample_jobs/hikey960-oe-aep.yaml')
+        job.validate()
+        description_ref = self.pipeline_reference('hikey960-oe-aep.yaml', job=job)
+        self.assertEqual(description_ref, job.pipeline.describe(False))
+
+        lxc_deploy = [action for action in job.pipeline.actions if action.name == 'lxc-deploy'][0]
+        lxc_overlay = [action for action in lxc_deploy.internal_pipeline.actions if action.name == 'lava-overlay'][0]
+        lxc_defs = [action for action in lxc_overlay.internal_pipeline.actions if action.name == 'test-definition'][0]
+        lxc_installscript = [action for action in lxc_defs.internal_pipeline.actions if action.name == 'test-install-overlay'][0]
+        fastboot_deploy = [action for action in job.pipeline.actions if action.name == 'fastboot-deploy'][0]
+        fastboot_overlay = [action for action in fastboot_deploy.internal_pipeline.actions if action.name == 'lava-overlay'][0]
+        fastboot_defs = [action for action in fastboot_overlay.internal_pipeline.actions if action.name == 'test-definition'][0]
+        fastboot_installscript = [action for action in fastboot_defs.internal_pipeline.actions if action.name == 'test-install-overlay'][0]
+
+        self.assertIn('distro', lxc_installscript.parameters['deployment_data'].keys())
+        self.assertIn('distro', list(lxc_installscript.parameters['deployment_data'].keys()))
+        self.assertIn('distro', list(lxc_installscript.parameters['deployment_data']))
+        self.assertIn('distro', dict(lxc_installscript.parameters['deployment_data']))
+        self.assertEqual('debian', lxc_installscript.parameters['deployment_data']['distro'])
+
+        self.assertIn('distro', fastboot_installscript.parameters['deployment_data'].keys())
+        self.assertIn('distro', list(fastboot_installscript.parameters['deployment_data'].keys()))
+        self.assertIn('distro', fastboot_installscript.parameters['deployment_data'])
+        self.assertIn('distro', dict(fastboot_installscript.parameters['deployment_data']))
+        self.assertEqual('oe', fastboot_installscript.parameters['deployment_data']['distro'])
