@@ -47,6 +47,7 @@ from lava_dispatcher.utils.messages import LinuxKernelMessages
 from lava_dispatcher.test.test_defs import allow_missing_path, check_missing_path
 from lava_dispatcher.test.utils import DummyLogger, infrastructure_error
 from lava_dispatcher.utils.strings import substitute
+from lava_dispatcher.connections.serial import QemuSession
 
 # pylint: disable=invalid-name
 
@@ -161,8 +162,8 @@ class TestKVMBasicDeploy(StdoutTestCase):  # pylint: disable=too-many-public-met
 
     def setUp(self):
         super().setUp()
-        factory = Factory()
-        self.job = factory.create_kvm_job('sample_jobs/kvm.yaml')
+        self.factory = Factory()
+        self.job = self.factory.create_job('qemu01.jinja2', 'sample_jobs/kvm.yaml')
 
     def test_deploy_job(self):
         self.assertEqual(self.job.pipeline.job, self.job)
@@ -185,6 +186,13 @@ class TestKVMBasicDeploy(StdoutTestCase):  # pylint: disable=too-many-public-met
         for action in self.job.pipeline.actions:
             self.assertEqual([], action.errors)
 
+    def test_available_architectures(self):
+        job_ctx = {'arch': 'unknown'}
+        job = self.factory.create_job('qemu01.jinja2', 'sample_jobs/kvm.yaml', job_ctx)
+        self.assertIsNotNone(job.device['available_architectures'])
+        self.assertEqual(job.parameters['context']['arch'], 'unknown')
+        self.assertRaises(JobError, job.pipeline.validate_actions)
+
     def test_overlay(self):
         overlay = None
         for action in self.job.pipeline.actions:
@@ -211,7 +219,7 @@ class TestKVMBasicDeploy(StdoutTestCase):  # pylint: disable=too-many-public-met
                 # get the action & populate it
                 self.assertEqual(action.parameters['method'], 'qemu')
                 self.assertEqual(action.parameters['prompts'], ['linaro-test', 'root@debian:~#'])
-                params = action.parameters.get('auto_login', None)
+                params = action.parameters.get('auto_login')
 
                 if 'login_prompt' in params:
                     self.assertEqual(params['login_prompt'], 'login:')
@@ -225,6 +233,32 @@ class TestKVMBasicDeploy(StdoutTestCase):  # pylint: disable=too-many-public-met
                 self.assertEqual(len(action.parameters['definitions']), 2)
 
 
+class TestKVMPortable(StdoutTestCase):  # pylint: disable=too-many-public-methods
+
+    def setUp(self):
+        super().setUp()
+        factory = Factory()
+        self.job = factory.create_kvm_job('sample_jobs/kvm-noos.yaml')
+
+    def test_deploy_job(self):
+        self.assertEqual(self.job.pipeline.job, self.job)
+        for action in self.job.pipeline.actions:
+            if isinstance(action, DeployAction):
+                self.assertEqual(action.job, self.job)
+
+    def test_pipeline(self):
+        description_ref = self.pipeline_reference('kvm-noos.yaml')
+        self.assertEqual(description_ref, self.job.pipeline.describe(False))
+
+    def test_validate(self):
+        try:
+            allow_missing_path(self.job.pipeline.validate_actions, self, 'qemu-system-x86_64')
+        except JobError as exc:
+            self.fail(exc)
+        for action in self.job.pipeline.actions:
+            self.assertEqual([], action.errors)
+
+
 class TestKVMQcow2Deploy(StdoutTestCase):  # pylint: disable=too-many-public-methods
 
     def setUp(self):
@@ -303,11 +337,11 @@ class TestKVMInlineTestDeploy(StdoutTestCase):  # pylint: disable=too-many-publi
 
     def test_extra_options(self):
         (rendered, _) = self.factory.create_device('kvm01.jinja2')
-        device = NewDevice(yaml.load(rendered))
+        device = NewDevice(yaml.safe_load(rendered))
         kvm_yaml = os.path.join(os.path.dirname(__file__), 'sample_jobs/kvm-inline.yaml')
         with open(kvm_yaml) as sample_job_data:
-            job_data = yaml.load(sample_job_data)
-        device['actions']['boot']['methods']['qemu']['parameters']['extra'] = yaml.load("""
+            job_data = yaml.safe_load(sample_job_data)
+        device['actions']['boot']['methods']['qemu']['parameters']['extra'] = yaml.safe_load("""
                   - -smp
                   - 1
                   - -global
@@ -357,7 +391,7 @@ class TestKVMInlineTestDeploy(StdoutTestCase):  # pylint: disable=too-many-publi
         yaml_file = os.path.join(location, '0/tests/0_smoke-tests-inline/inline/smoke-tests-basic.yaml')
         self.assertTrue(os.path.exists(yaml_file))
         with open(yaml_file, 'r') as f_in:
-            testdef = yaml.load(f_in)
+            testdef = yaml.safe_load(f_in)
         expected_testdef = {'metadata':
                             {'description': 'Basic system test command for Linaro Ubuntu images',
                              'devices': ['panda', 'panda-es', 'arndale', 'vexpress-a9', 'vexpress-tc2'],
@@ -378,6 +412,29 @@ class TestKVMInlineTestDeploy(StdoutTestCase):  # pylint: disable=too-many-publi
         self.assertEqual(set(testdef), set(expected_testdef))
 
 
+class TestKvmConnection(StdoutTestCase):
+
+    def setUp(self):
+        super().setUp()
+        factory = Factory()
+        self.job = factory.create_kvm_job('sample_jobs/qemu-reboot.yaml')
+        self.job.logger = DummyLogger()
+        self.max_end_time = time.time() + 30
+
+    def test_kvm_connection(self):
+        self.job.validate()
+        description_ref = self.pipeline_reference('qemu-reboot.yaml')
+        self.assertEqual(description_ref, self.job.pipeline.describe(False))
+        bootaction = [action for action in self.job.pipeline.actions if action.name == 'boot-image-retry'][0]
+        bootqemu = [action for action in bootaction.internal_pipeline.actions if action.name == 'boot-qemu-image'][0]
+        call_qemu = [action for action in bootqemu.internal_pipeline.actions if action.name == 'execute-qemu'][0]
+        self.assertEqual(call_qemu.session_class, QemuSession)
+        bootaction = [action for action in self.job.pipeline.actions if action.name == 'boot-image-retry'][1]
+        bootqemu = [action for action in bootaction.internal_pipeline.actions if action.name == 'boot-qemu-image'][0]
+        call_qemu = [action for action in bootqemu.internal_pipeline.actions if action.name == 'execute-qemu'][0]
+        self.assertEqual(call_qemu.session_class, QemuSession)
+
+
 class TestAutoLogin(StdoutTestCase):
 
     def setUp(self):
@@ -408,10 +465,8 @@ class TestAutoLogin(StdoutTestCase):
         conn = autologinaction.run(shell_connection, max_end_time=self.max_end_time)
         self.assertEqual(shell_connection.raw_connection.linesep, 'testsep')
 
-        self.assertIn('lava-test: # ', conn.prompt_str)
         self.assertIn('root@debian:~#', conn.prompt_str)
         conn.prompt_str = 'root@stretch:'
-        self.assertNotIn('lava-test: # ', conn.prompt_str)
         self.assertNotIn('root@debian:~#', conn.prompt_str)
         self.assertIn('root@stretch:', conn.prompt_str)
 
@@ -482,7 +537,6 @@ class TestAutoLogin(StdoutTestCase):
         # Test the AutoLoginAction directly
         conn = autologinaction.run(shell_connection, max_end_time=self.max_end_time)
 
-        self.assertIn('lava-test: # ', conn.prompt_str)
         self.assertIn('root@debian:~#', conn.prompt_str)
 
     def test_missing_autologin_void_prompts_str(self):
@@ -513,7 +567,6 @@ class TestAutoLogin(StdoutTestCase):
         # Test the AutoLoginAction directly
         conn = autologinaction.run(shell_connection, max_end_time=self.max_end_time)
 
-        self.assertIn('lava-test: # ', conn.prompt_str)
         self.assertIn('root@debian:~#', conn.prompt_str)
 
     def test_autologin_login_incorrect(self):
@@ -536,7 +589,6 @@ class TestAutoLogin(StdoutTestCase):
         # Test the AutoLoginAction directly
         conn = autologinaction.run(shell_connection, max_end_time=self.max_end_time)
 
-        self.assertIn('lava-test: # ', conn.prompt_str)
         self.assertIn('root@debian:~#', conn.prompt_str)
         self.assertIn('Login incorrect', conn.prompt_str)
         self.assertIn('Login timed out', conn.prompt_str)
@@ -646,7 +698,7 @@ class TestChecksum(StdoutTestCase):
 
     def test_uboot_checksum(self):
         (rendered, _) = self.factory.create_device('bbb-01.jinja2')
-        device = NewDevice(yaml.load(rendered))
+        device = NewDevice(yaml.safe_load(rendered))
         bbb_yaml = os.path.join(os.path.dirname(__file__), 'sample_jobs/bbb-ramdisk-nfs.yaml')
         with open(bbb_yaml) as sample_job_data:
             parser = JobParser()
@@ -655,9 +707,9 @@ class TestChecksum(StdoutTestCase):
         download = [action for action in deploy.internal_pipeline.actions if action.name == 'download-retry'][0]
         helper = [action for action in download.internal_pipeline.actions if action.name == 'file-download'][0]
         remote = helper.parameters[helper.key]
-        md5sum = remote.get('md5sum', None)
+        md5sum = remote.get('md5sum')
         self.assertIsNone(md5sum)
-        sha256sum = remote.get('sha256sum', None)
+        sha256sum = remote.get('sha256sum')
         self.assertIsNotNone(sha256sum)
 
 
