@@ -170,7 +170,7 @@ class TestMultinode(StdoutTestCase):  # pylint: disable=too-many-public-methods
         self.assertEqual(client_protocol.parameters['protocols'][client_protocol.name]['roles']['kvm02'], 'server')
         self.assertEqual(server_protocol.parameters['protocols'][client_protocol.name]['roles']['kvm01'], 'client')
         self.assertEqual(server_protocol.parameters['protocols'][client_protocol.name]['roles']['kvm02'], 'server')
-        self.assertEqual(client_multinode.lava_multi_node_cache_file, '/tmp/lava_multi_node_cache.txt')
+        self.assertEqual(client_multinode.lava_multi_node_cache_file, '/tmp/lava_multi_node_cache.txt')  # nosec - replicating DUT behaviour.
         self.assertIsNotNone(client_multinode.lava_multi_node_test_dir)
         self.assertTrue(os.path.exists(client_multinode.lava_multi_node_test_dir))
 
@@ -211,7 +211,7 @@ class TestMultinode(StdoutTestCase):  # pylint: disable=too-many-public-methods
         for action in self.client_job.pipeline.actions:
             data = action.explode()
             data_str = yaml.dump(data)
-            yaml.load(data_str)
+            yaml.load(data_str)  # nosec not suitable for safe_load
 
     def test_multinode_timeout(self):
         """
@@ -464,11 +464,11 @@ class TestMultinode(StdoutTestCase):  # pylint: disable=too-many-public-methods
                 'action': 'execute-qemu',
                 'message': {
                     'ipv4': '$IPV4',
-                    'yaml_line': 65
+                    'yaml_line': 23
                 },
                 'messageID': 'test',
                 'request': 'lava-wait',
-                'yaml_line': 62
+                'yaml_line': 22
             }])
         client_calls = {}
         for action in retry.internal_pipeline.actions:
@@ -514,9 +514,9 @@ class TestMultinode(StdoutTestCase):  # pylint: disable=too-many-public-methods
                 'action': 'execute-qemu',
                 'message': {
                     'ipv4': reply['message'][mn_protocol.job_id]['ipv4'],
-                    'yaml_line': 65
+                    'yaml_line': 23
                 },
-                'yaml_line': 62,
+                'yaml_line': 22,
                 'request': 'lava-wait',
                 'messageID': 'test'
             }
@@ -574,7 +574,7 @@ class TestProtocol(StdoutTestCase):  # pylint: disable=too-many-public-methods
         self.assertTrue(self.coord.group['group'] == '')
         self.coord.conn.clearPasses()
 
-    class FakeClient(object):
+    class FakeClient:
         """
         acts as the client socket, passing data to the fake coordinator dataReceived() call
         """
@@ -589,7 +589,7 @@ class TestProtocol(StdoutTestCase):  # pylint: disable=too-many-public-methods
         def send(self, msg):
             if self.header:
                 self.header = False
-                assert(int(msg, 16) < 0xFFFE)
+                assert(int(msg, 16) < 0xFFFE)  # nosec - unit test support
             else:
                 message = json.loads(msg)
                 self.coord.dataReceived(message)
