@@ -22,7 +22,7 @@ import os
 import yaml
 import unittest
 from lava_dispatcher.test.test_basic import Factory, StdoutTestCase
-from lava_dispatcher.action import JobError
+from lava_common.exceptions import JobError
 from lava_dispatcher.device import NewDevice
 from lava_dispatcher.parser import JobParser
 from lava_dispatcher.actions.boot import BootloaderSecondaryMedia
@@ -41,8 +41,8 @@ class RemovableFactory(Factory):  # pylint: disable=too-few-public-methods
 
     def create_job(self, sample_job, device_file):  # pylint: disable=no-self-use
         device = NewDevice(os.path.join(os.path.dirname(__file__), device_file))
-        yaml = os.path.join(os.path.dirname(__file__), sample_job)
-        with open(yaml) as sample_job_data:
+        j_yaml = os.path.join(os.path.dirname(__file__), sample_job)
+        with open(j_yaml) as sample_job_data:
             parser = JobParser()
             job = parser.parse(sample_job_data, device, 4212, None, "")
         job.logger = DummyLogger()
@@ -60,15 +60,15 @@ class TestRemovable(StdoutTestCase):  # pylint: disable=too-many-public-methods
         Test that the correct parameters have been set for the device
         """
         (rendered, _) = self.factory.create_device('cubie2.jinja2')
-        cubie = NewDevice(yaml.load(rendered))
-        self.assertIsNotNone(cubie['parameters']['media'].get('usb', None))
-        self.assertIsNotNone(cubie.get('commands', None))
-        self.assertIsNotNone(cubie.get('actions', None))
-        self.assertIsNotNone(cubie['actions'].get('deploy', None))
-        self.assertIsNotNone(cubie['actions']['deploy'].get('methods', None))
+        cubie = NewDevice(yaml.safe_load(rendered))
+        self.assertIsNotNone(cubie['parameters']['media'].get('usb'))
+        self.assertIsNotNone(cubie.get('commands'))
+        self.assertIsNotNone(cubie.get('actions'))
+        self.assertIsNotNone(cubie['actions'].get('deploy'))
+        self.assertIsNotNone(cubie['actions']['deploy'].get('methods'))
         self.assertIn('usb', cubie['actions']['deploy']['methods'])
-        self.assertIsNotNone(cubie['actions'].get('boot', None))
-        self.assertIsNotNone(cubie['actions']['boot'].get('methods', None))
+        self.assertIsNotNone(cubie['actions'].get('boot'))
+        self.assertIsNotNone(cubie['actions']['boot'].get('methods'))
         self.assertIn('u-boot', cubie['actions']['boot']['methods'])
         u_boot_params = cubie['actions']['boot']['methods']['u-boot']
         self.assertIn('usb', u_boot_params)
@@ -113,7 +113,7 @@ class TestRemovable(StdoutTestCase):  # pylint: disable=too-many-public-methods
         Test that the job parameters match expected structure
         """
         (rendered, _) = self.factory.create_device('cubie1.jinja2')
-        cubie = NewDevice(yaml.load(rendered))
+        cubie = NewDevice(yaml.safe_load(rendered))
         job = self._check_valid_job(cubie, 'cubietruck-removable.yaml')
         self._check_job_parameters(cubie, job, 'download')
 
@@ -122,7 +122,7 @@ class TestRemovable(StdoutTestCase):  # pylint: disable=too-many-public-methods
         Test that the job parameters with a writer tool match expected structure
         """
         (rendered, _) = self.factory.create_device('cubie1.jinja2')
-        cubie = NewDevice(yaml.load(rendered))
+        cubie = NewDevice(yaml.safe_load(rendered))
         job = self._check_valid_job(cubie, 'cubietruck-removable-with-writer.yaml')
         self._check_job_parameters(cubie, job, 'writer')
 
@@ -159,12 +159,12 @@ class TestRemovable(StdoutTestCase):  # pylint: disable=too-many-public-methods
 
     def test_deployment(self):
         (rendered, _) = self.factory.create_device('cubie1.jinja2')
-        cubie = NewDevice(yaml.load(rendered))
+        cubie = NewDevice(yaml.safe_load(rendered))
         self._check_deployment(cubie, 'cubietruck-removable.yaml')
 
     def test_writer_deployment(self):
         (rendered, _) = self.factory.create_device('cubie1.jinja2')
-        cubie = NewDevice(yaml.load(rendered))
+        cubie = NewDevice(yaml.safe_load(rendered))
         self._check_deployment(cubie, 'cubietruck-removable-with-writer.yaml')
 
     def test_juno_deployment(self):
@@ -243,7 +243,7 @@ class TestRemovable(StdoutTestCase):  # pylint: disable=too-many-public-methods
         """
         job_parser = JobParser()
         (rendered, _) = self.factory.create_device('bbb-01.jinja2')
-        bbb = NewDevice(yaml.load(rendered))
+        bbb = NewDevice(yaml.safe_load(rendered))
         sample_job_file = os.path.join(os.path.dirname(__file__), 'sample_jobs/uboot-ramdisk.yaml')
         with open(sample_job_file) as sample_job_data:
             job = job_parser.parse(sample_job_data, bbb, 4212, None, "")
@@ -262,7 +262,7 @@ class TestRemovable(StdoutTestCase):  # pylint: disable=too-many-public-methods
         """
         job_parser = JobParser()
         (rendered, _) = self.factory.create_device('cubie1.jinja2')
-        cubie = NewDevice(yaml.load(rendered))
+        cubie = NewDevice(yaml.safe_load(rendered))
         sample_job_file = os.path.join(os.path.dirname(__file__), 'sample_jobs/cubietruck-removable.yaml')
         with open(sample_job_file) as sample_job_data:
             job = job_parser.parse(sample_job_data, cubie, 4212, None, "")
@@ -290,7 +290,7 @@ class TestRemovable(StdoutTestCase):  # pylint: disable=too-many-public-methods
         device_id = u_boot_action.get_namespace_data(action='storage-deploy', label='u-boot', key='device')
         self.assertIsNotNone(device_id)
         kernel_type = u_boot_action.parameters['kernel_type']
-        bootcommand = map_kernel_uboot(kernel_type, device_params=cubie.get('parameters', None))
+        bootcommand = map_kernel_uboot(kernel_type, device_params=cubie.get('parameters'))
         substitutions = {
             '{BOOTX}': "%s %s %s %s" % (
                 bootcommand,
