@@ -33,7 +33,8 @@ from lava_dispatcher.actions.boot import BootloaderCommandOverlay
 from lava_dispatcher.actions.deploy.apply_overlay import CompressRamdisk
 from lava_dispatcher.actions.deploy.tftp import TftpAction
 from lava_dispatcher.job import Job
-from lava_dispatcher.action import Pipeline, JobError
+from lava_dispatcher.action import Pipeline
+from lava_common.exceptions import JobError
 from lava_dispatcher.test.test_basic import Factory, StdoutTestCase
 from lava_dispatcher.test.utils import DummyLogger, infrastructure_error
 from lava_dispatcher.utils.network import dispatcher_ip
@@ -76,7 +77,6 @@ class TestUbootAction(StdoutTestCase):  # pylint: disable=too-many-public-method
         self.assertEqual(description_ref, job.pipeline.describe(False))
 
         self.assertIsNone(job.validate())
-        self.assertEqual(job.device['device_type'], 'beaglebone-black')
 
     def test_tftp_pipeline(self):
         job = self.factory.create_bbb_job('sample_jobs/uboot-ramdisk.yaml')
@@ -108,7 +108,7 @@ class TestUbootAction(StdoutTestCase):  # pylint: disable=too-many-public-method
         self.assertEqual(job.device['commands'].get('interrupt', ' '), ' ')
         methods = job.device['actions']['boot']['methods']
         self.assertIn('u-boot', methods)
-        self.assertEqual(methods['u-boot']['parameters'].get('bootloader_prompt', None), 'U-Boot')
+        self.assertEqual(methods['u-boot']['parameters'].get('bootloader_prompt'), 'U-Boot')
 
     @unittest.skipIf(infrastructure_error('mkimage'), "u-boot-tools not installed")
     def test_uboot_action(self):
@@ -361,7 +361,7 @@ class TestUbootAction(StdoutTestCase):  # pylint: disable=too-many-public-method
         """
         job_parser = JobParser()
         (rendered, _) = self.factory.create_device('cubie1.jinja2')
-        cubie = NewDevice(yaml.load(rendered))
+        cubie = NewDevice(yaml.safe_load(rendered))
         sample_job_file = os.path.join(os.path.dirname(__file__), 'sample_jobs/cubietruck-removable.yaml')
         sample_job_data = open(sample_job_file)
         job = job_parser.parse(sample_job_data, cubie, 4212, None, "")
@@ -419,6 +419,18 @@ class TestUbootAction(StdoutTestCase):  # pylint: disable=too-many-public-method
         description_ref = self.pipeline_reference('zcu102-ramdisk.yaml', job=job)
         self.assertEqual(description_ref, job.pipeline.describe(False))
 
+    def test_imx8m(self):
+        job = self.factory.create_job('imx8m-01.jinja2', 'sample_jobs/imx8m.yaml')
+        self.assertIsNotNone(job)
+        job.validate()
+        self.assertEqual(job.pipeline.errors, [])
+        description_ref = self.pipeline_reference('imx8m.yaml', job=job)
+        self.assertEqual(description_ref, job.pipeline.describe(False))
+        deploy = [action for action in job.pipeline.actions if action.name == 'fastboot-deploy'][0]
+        fastboot = [action for action in deploy.internal_pipeline.actions if action.name == 'uboot-enter-fastboot'][0]
+        bootloader = [action for action in fastboot.internal_pipeline.actions if action.name == 'bootloader-interrupt'][0]
+        self.assertEqual('u-boot', bootloader.method)
+
 
 class TestKernelConversion(StdoutTestCase):
 
@@ -432,7 +444,7 @@ class TestKernelConversion(StdoutTestCase):
         self.device = NewDevice(os.path.join(os.path.dirname(__file__), '../devices/bbb-01.yaml'))
         bbb_yaml = os.path.join(os.path.dirname(__file__), 'sample_jobs/uboot-ramdisk.yaml')
         with open(bbb_yaml) as sample_job_data:
-            self.base_data = yaml.load(sample_job_data)
+            self.base_data = yaml.safe_load(sample_job_data)
         self.deploy_block = [block for block in self.base_data['actions'] if 'deploy' in block][0]['deploy']
         self.boot_block = [block for block in self.base_data['actions'] if 'boot' in block][0]['boot']
         self.parser = JobParser()
@@ -518,16 +530,16 @@ class TestOverlayCommands(StdoutTestCase):  # pylint: disable=too-many-public-me
         ramdisk = [action for action in prepare.internal_pipeline.actions if action.name == 'extract-overlay-ramdisk'][0]
         modules = [action for action in prepare.internal_pipeline.actions if action.name == 'extract-modules'][0]
         overlay = [action for action in prepare.internal_pipeline.actions if action.name == 'apply-overlay-tftp'][0]
-        self.assertIsNotNone(ramdisk.parameters.get('ramdisk'), None)
-        self.assertIsNotNone(ramdisk.parameters['ramdisk'].get('url'), None)
-        self.assertIsNotNone(ramdisk.parameters['ramdisk'].get('compression'), None)
+        self.assertIsNotNone(ramdisk.parameters.get('ramdisk'))
+        self.assertIsNotNone(ramdisk.parameters['ramdisk'].get('url'))
+        self.assertIsNotNone(ramdisk.parameters['ramdisk'].get('compression'))
         self.assertTrue(ramdisk.parameters['ramdisk'].get('install_modules', True))
         self.assertTrue(ramdisk.parameters['ramdisk'].get('install_overlay', True))
-        self.assertIsNotNone(modules.parameters.get('ramdisk', None))
-        self.assertIsNotNone(modules.parameters.get('nfsrootfs', None))
-        self.assertIsNotNone(nfs.parameters.get('nfsrootfs', None))
-        self.assertIsNotNone(overlay.parameters.get('nfsrootfs', None))
-        self.assertIsNotNone(overlay.parameters.get('ramdisk', None))
+        self.assertIsNotNone(modules.parameters.get('ramdisk'))
+        self.assertIsNotNone(modules.parameters.get('nfsrootfs'))
+        self.assertIsNotNone(nfs.parameters.get('nfsrootfs'))
+        self.assertIsNotNone(overlay.parameters.get('nfsrootfs'))
+        self.assertIsNotNone(overlay.parameters.get('ramdisk'))
 
     def test_ramdisk_nfs_nomodules(self):
         job = self.factory.create_bbb_job('sample_jobs/bbb-uinitrd-nfs.yaml')
@@ -537,12 +549,12 @@ class TestOverlayCommands(StdoutTestCase):  # pylint: disable=too-many-public-me
         ramdisk = [action for action in prepare.internal_pipeline.actions if action.name == 'extract-overlay-ramdisk'][0]
         modules = [action for action in prepare.internal_pipeline.actions if action.name == 'extract-modules'][0]
         overlay = [action for action in prepare.internal_pipeline.actions if action.name == 'apply-overlay-tftp'][0]
-        self.assertIsNotNone(ramdisk.parameters.get('ramdisk'), None)
-        self.assertIsNotNone(ramdisk.parameters['ramdisk'].get('url'), None)
-        self.assertIsNone(ramdisk.parameters['ramdisk'].get('compression'), None)
+        self.assertIsNotNone(ramdisk.parameters.get('ramdisk'))
+        self.assertIsNotNone(ramdisk.parameters['ramdisk'].get('url'))
+        self.assertIsNone(ramdisk.parameters['ramdisk'].get('compression'))
         self.assertFalse(ramdisk.parameters['ramdisk'].get('install_overlay', True))
-        self.assertIsNotNone(modules.parameters.get('ramdisk', None))
-        self.assertIsNotNone(modules.parameters.get('nfsrootfs', None))
-        self.assertIsNotNone(nfs.parameters.get('nfsrootfs', None))
-        self.assertIsNotNone(overlay.parameters.get('nfsrootfs', None))
-        self.assertIsNotNone(overlay.parameters.get('ramdisk', None))
+        self.assertIsNotNone(modules.parameters.get('ramdisk'))
+        self.assertIsNotNone(modules.parameters.get('nfsrootfs'))
+        self.assertIsNotNone(nfs.parameters.get('nfsrootfs'))
+        self.assertIsNotNone(overlay.parameters.get('nfsrootfs'))
+        self.assertIsNotNone(overlay.parameters.get('ramdisk'))
