@@ -1,22 +1,21 @@
-# Copyright (C) 2017 Linaro Limited
+# -*- coding: utf-8 -*-
+# Copyright (C) 2017-2018 Linaro Limited
 #
 # Author: Stevan Radakovic <stevan.radakovic@linaro.org>
 #
-# This file is part of LAVA Server.
+# This file is part of LAVA.
 #
-# LAVA Server is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
+# LAVA is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License version 3
+# as published by the Free Software Foundation
 #
-# LAVA Server is distributed in the hope that it will be useful,
+# LAVA is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License along
-# with this program; if not, see <http://www.gnu.org/licenses>.
-
+# You should have received a copy of the GNU Affero General Public License
+# along with LAVA.  If not, see <http://www.gnu.org/licenses/>.
 
 import sys
 from django.core.management.base import BaseCommand
@@ -33,10 +32,7 @@ class Command(BaseCommand):
     """
 
     logger = None
-
-    def __init__(self, *args, **options):
-        super(Command, self).__init__(*args, **options)
-        self.help = "Drop materialized views from database"
+    help = "Drop materialized views from database"
 
     def handle(self, *args, **options):
         for query in Query.objects.all().filter(is_live=False):
