@@ -1,3 +1,23 @@
+# -*- coding: utf-8 -*-
+# Copyright (C) 2017-2018 Linaro Limited
+#
+# Author: Neil Williams <neil.williams@linaro.org>
+#         Senthil Kumaran <senthil.kumaran@linaro.org>
+#
+# This file is part of LAVA.
+#
+# LAVA is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License version 3
+# as published by the Free Software Foundation
+#
+# LAVA is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU Affero General Public License
+# along with LAVA.  If not, see <http://www.gnu.org/licenses/>.
+
 from functools import wraps
 from simplejson import JSONDecodeError
 import yaml
@@ -20,7 +40,8 @@ from lava_scheduler_app.views import (
 )
 from lava_scheduler_app.dbutils import (
     device_type_summary,
-    testjob_submission
+    testjob_submission,
+    active_device_types,
 )
 from lava_scheduler_app.schema import (
     validate_submission,
@@ -35,18 +56,20 @@ from lava_scheduler_app.schema import (
 # pylint: disable=no-self-use
 
 
-def check_superuser(f):
-    """ decorator to check that the caller is a super-user """
-    @wraps(f)
-    def wrapper(self, *args, **kwargs):
-        self._authenticate()
-        if not self.user.is_superuser:
-            raise xmlrpc.client.Fault(
-                403,
-                "User '%s' is not superuser." % self.user.username
-            )
-        return f(self, *args, **kwargs)
-    return wrapper
+def check_perm(perm):
+    """ decorator to check that the caller has the given permission """
+    def decorator(f):
+        @wraps(f)
+        def wrapper(self, *args, **kwargs):
+            self._authenticate()
+            if not self.user.has_perm(perm):
+                raise xmlrpc.client.Fault(
+                    403,
+                    "User '%s' is missing permission %s ." % (self.user.username, perm)
+                )
+            return f(self, *args, **kwargs)
+        return wrapper
+    return decorator
 
 
 def build_device_status_display(state, health):
@@ -105,7 +128,7 @@ class SchedulerAPI(ExposedAPI):
             raise xmlrpc.client.Fault(404, "Specified device or device type not found.")
         except DevicesUnavailableException as exc:
             raise xmlrpc.client.Fault(400, "Device unavailable: %s" % str(exc))
-        if isinstance(job, type(list())):
+        if isinstance(job, list):
             return [j.sub_id for j in job]
         else:
             return job.id
@@ -421,11 +444,14 @@ class SchedulerAPI(ExposedAPI):
             job_qs = job_qs.filter(submitter=self.user)
         job_list = []
         for job in job_qs.all()[:count]:
+            hostname = ''
+            if job.actual_device:
+                hostname = job.actual_device.hostname
             job_dict = {
                 "id": job.id,
                 "description": job.description,
                 "status": job.get_legacy_status_display(),
-                "device": job.actual_device.hostname,
+                "device": hostname,
             }
             if not job.can_view(self.user):
                 job_dict["id"] = None
@@ -711,7 +737,7 @@ class SchedulerAPI(ExposedAPI):
                     403, "Permission denied for user to put %s into online mode." % hostname
                 )
 
-    def pending_jobs_by_device_type(self):
+    def pending_jobs_by_device_type(self, all=False):
         """
         Name
         ----
@@ -720,10 +746,12 @@ class SchedulerAPI(ExposedAPI):
         Description
         -----------
         Get number of pending jobs in each device type.
+        Private test jobs and hidden device types are
+        excluded, except for authenticated superusers.
 
         Arguments
         ---------
-        None
+        `all`: boolean - include retired devices and undisplayed device-types in the listing.
 
         Return value
         ------------
@@ -736,19 +764,31 @@ class SchedulerAPI(ExposedAPI):
 
         pending_jobs_by_device = {}
 
-        jobs_res = TestJob.objects.filter(state=TestJob.STATE_SUBMITTED) \
-                                  .values_list('requested_device_type_id')\
-                                  .annotate(pending_jobs=(Count('id')))
+        jobs_res = TestJob.objects.filter(state=TestJob.STATE_SUBMITTED)
+        jobs_res = jobs_res.exclude(requested_device_type_id__isnull=True)
+
+        if not self.user or not self.user.is_superuser:
+            jobs_res = jobs_res.filter(is_public=True)
+
+        jobs_res = jobs_res.values_list('requested_device_type_id')
+        jobs_res = jobs_res.annotate(pending_jobs=(Count('id')))
+
         jobs = {}
         jobs_hash = dict(jobs_res)
         for job in jobs_hash:
-            if job:
-                jobs[job] = jobs_hash[job]
+            jobs[job] = jobs_hash[job]
         pending_jobs_by_device.update(jobs)
 
         # Get rest of the devices and put number of pending jobs as 0.
-        device_types = DeviceType.objects.values_list('name', flat=True)
-        for device_type in device_types:
+        if all:
+            device_types = DeviceType.objects.all()
+        else:
+            device_types = active_device_types()
+
+        if not self.user or not self.user.is_superuser:
+            device_types = device_types.filter(owners_only=False)
+
+        for device_type in device_types.values_list('name', flat=True):
             if device_type not in pending_jobs_by_device:
                 pending_jobs_by_device[device_type] = 0
 
@@ -1063,7 +1103,7 @@ class SchedulerAPI(ExposedAPI):
         ------------
         This function returns an XML-RPC binary data of output file.
         """
-        return get_device_config(device_hostname, context)
+        return self.get_device_config(device_hostname)
 
     def get_device_config(self, device_hostname, context=None):
         """
