@@ -1,21 +1,23 @@
-# Copyright (C) 2017 Linaro Limited
+# -*- coding: utf-8 -*-
+# Copyright (C) 2017-2018 Linaro Limited
 #
 # Author: Remi Duraffort <remi.duraffort@linaro.org>
 #
-# This file is part of Lava Server.
+# This file is part of LAVA.
 #
-# Lava Server is free software: you can redistribute it and/or modify
+# LAVA is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License version 3
 # as published by the Free Software Foundation
 #
-# Lava Server is distributed in the hope that it will be useful,
+# LAVA is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with Lava Server.  If not, see <http://www.gnu.org/licenses/>.
+# along with LAVA.  If not, see <http://www.gnu.org/licenses/>.
 
+import contextlib
 import errno
 import glob
 import os
@@ -24,7 +26,7 @@ import xmlrpc.client
 from django.db import IntegrityError
 
 from linaro_django_xmlrpc.models import ExposedV2API
-from lava_scheduler_app.api import check_superuser
+from lava_scheduler_app.api import check_perm
 from lava_scheduler_app.models import Alias, DeviceType
 
 
@@ -40,7 +42,7 @@ class SchedulerDeviceTypesAPI(ExposedV2API):
         available_types.sort()
         return available_types
 
-    @check_superuser
+    @check_perm("lava_scheduler_app.add_devicetype")
     def add(self, name, description, display, owners_only,
             health_frequency, health_denominator):
         """
@@ -116,12 +118,10 @@ class SchedulerDeviceTypesAPI(ExposedV2API):
         ------------
         The health-check
         """
-        try:
+        with contextlib.suppress(DeviceType.DoesNotExist):
             dt = DeviceType.objects.get(name=name)
             if dt.owners_only and not dt.some_devices_visible_to(self.user):
                 raise xmlrpc.client.Fault(404, "Device-type '%s' was not found." % name)
-        except DeviceType.DoesNotExist:
-            pass
 
         # Filename should not be a path or starting with a dot
         if os.path.basename(name) != name or name[0] == ".":
@@ -132,7 +132,7 @@ class SchedulerDeviceTypesAPI(ExposedV2API):
             filename += ".yaml" if not filename.endswith('.yaml') else ''
             with open(filename, "r") as f_in:
                 return xmlrpc.client.Binary(f_in.read().encode("utf-8"))
-        except IOError as exc:
+        except OSError as exc:
             if exc.errno == errno.ENOENT:
                 raise xmlrpc.client.Fault(404, "Device-type '%s' was not found." % name)
             else:
@@ -163,12 +163,10 @@ class SchedulerDeviceTypesAPI(ExposedV2API):
         ------------
         The device-type configuration
         """
-        try:
+        with contextlib.suppress(DeviceType.DoesNotExist):
             dt = DeviceType.objects.get(name=name)
             if dt.owners_only and not dt.some_devices_visible_to(self.user):
                 raise xmlrpc.client.Fault(404, "Device-type '%s' was not found." % name)
-        except DeviceType.DoesNotExist:
-            pass
 
         # Filename should not be a path or starting with a dot
         if os.path.basename(name) != name or name[0] == ".":
@@ -179,13 +177,13 @@ class SchedulerDeviceTypesAPI(ExposedV2API):
             filename += ".jinja2" if not filename.endswith('.jinja2') else ''
             with open(filename, "r") as f_in:
                 return xmlrpc.client.Binary(f_in.read().encode("utf-8"))
-        except IOError as exc:
+        except OSError as exc:
             if exc.errno == errno.ENOENT:
                 raise xmlrpc.client.Fault(404, "Device-type '%s' was not found." % name)
             else:
                 raise xmlrpc.client.Fault(400, "Unable to read device-type configuration: %s" % exc.strerror)
 
-    @check_superuser
+    @check_perm("lava_scheduler_app.change_devicetype")
     def set_health_check(self, name, config):
         """
         Name
@@ -223,11 +221,11 @@ class SchedulerDeviceTypesAPI(ExposedV2API):
             filename += ".yaml" if not filename.endswith('.yaml') else ''
             with open(filename, "w") as f_out:
                 f_out.write(config)
-        except IOError as exc:
+        except OSError as exc:
             raise xmlrpc.client.Fault(
                 400, "Unable to write health-check: %s" % exc.strerror)
 
-    @check_superuser
+    @check_perm("lava_scheduler_app.change_devicetype")
     def set_template(self, name, config):
         """
         Name
@@ -265,7 +263,7 @@ class SchedulerDeviceTypesAPI(ExposedV2API):
             filename += ".jinja2" if not filename.endswith('.jinja2') else ''
             with open(filename, "w") as f_out:
                 f_out.write(config)
-        except IOError as exc:
+        except OSError as exc:
             raise xmlrpc.client.Fault(
                 400, "Unable to write device-type configuration: %s" % exc.strerror)
 
@@ -350,7 +348,7 @@ class SchedulerDeviceTypesAPI(ExposedV2API):
 
         return dt_dict
 
-    @check_superuser
+    @check_perm("lava_scheduler_app.change_devicetype")
     def update(self, name, description, display, owners_only, health_frequency,
                health_denominator, health_disabled):
         """
@@ -425,7 +423,8 @@ class SchedulerDeviceTypesAPI(ExposedV2API):
 
 class SchedulerDeviceTypesAliasesAPI(ExposedV2API):
 
-    @check_superuser
+    @check_perm("lava_scheduler_app.add_alias")
+    @check_perm("lava_scheduler_app.change_devicetype")
     def add(self, name, alias):
         """
         Name
@@ -489,7 +488,7 @@ class SchedulerDeviceTypesAliasesAPI(ExposedV2API):
 
         return [a.name for a in dt.aliases.all().order_by("name")]
 
-    @check_superuser
+    @check_perm("lava_scheduler_app.change_devicetype")
     def delete(self, name, alias):
         """
         Name
