@@ -1,25 +1,21 @@
 # -*- coding: utf-8 -*-
-#
-# Copyright (C) 2017 Linaro Limited
+# Copyright (C) 2018 Linaro Limited
 #
 # Author: Remi Duraffort <remi.duraffort@linaro.org>
 #
-# This file is part of LAVA Dispatcher.
+# This file is part of LAVA.
 #
-# LAVA Dispatcher is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
+# LAVA is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License version 3
+# as published by the Free Software Foundation
 #
-# LAVA Dispatcher is distributed in the hope that it will be useful,
+# LAVA is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, see <http://www.gnu.org/licenses>.
-
-from __future__ import unicode_literals
+# You should have received a copy of the GNU Affero General Public License
+# along with LAVA.  If not, see <http://www.gnu.org/licenses/>.
 
 import datetime
 import yaml
@@ -33,6 +29,7 @@ from lava_scheduler_app.models import (
     DeviceType,
     Device,
     _create_pipeline_job,
+    _check_submit_to_device,
     TestJob,
     Worker
 )
@@ -50,13 +47,13 @@ def schedule_health_checks(logger, available_dt=None):
     jobs = []
     hc_disabled = []
     if available_dt:
-        query = DeviceType.objects.filter(name__in=available_dt)
+        query = DeviceType.objects.filter(name__in=available_dt, display=True)
     else:
-        query = DeviceType.objects.all()
+        query = DeviceType.objects.filter(display=True)
     for dt in query.order_by("name"):
         if dt.disable_health_check:
             hc_disabled.append(dt.name)
-            # Add all devices o that type to the list of available devices
+            # Add all devices of that type to the list of available devices
             devices = dt.device_set.filter(state=Device.STATE_IDLE)
             devices = devices.filter(worker_host__state=Worker.STATE_ONLINE)
             devices = devices.filter(health__in=[Device.HEALTH_GOOD,
@@ -126,6 +123,17 @@ def schedule_health_checks_for_device_type(logger, dt):
         logger.debug(" -> %s (%s, %s)", device.hostname,
                      device.get_state_display(),
                      device.get_health_display())
+        if not device.is_valid():
+            prev_health_display = device.get_health_display()
+            device.health = Device.HEALTH_BAD
+            device.log_admin_entry(
+                None, "%s → %s (Invalid device configuration)" % (
+                    prev_health_display, device.get_health_display()))
+            device.save()
+            logger.debug(
+                "%s → %s (Invalid device configuration for %s)" % (
+                    prev_health_display, device.get_health_display(), device.hostname))
+            continue
         logger.debug("  |--> scheduling health check")
         try:
             jobs.append(schedule_health_check(device, health_check))
@@ -143,9 +151,9 @@ def schedule_health_checks_for_device_type(logger, dt):
 
 def schedule_health_check(device, definition):
     user = User.objects.get(username="lava-health")
-    job = _create_pipeline_job(yaml.safe_load(definition), user, [],
-                               device_type=device.device_type,
-                               orig=definition, health_check=True)
+    job = _create_pipeline_job(
+        yaml.safe_load(definition), user, [], device=device, device_type=device.device_type,
+        orig=definition, health_check=True)
     job.go_state_scheduled(device)
     job.save()
     return job.id
@@ -206,6 +214,18 @@ def schedule_jobs_for_device(logger, device):
         if not job_tags.issubset(device_tags):
             continue
 
+        if not device.is_valid():
+            prev_health_display = device.get_health_display()
+            device.health = Device.HEALTH_BAD
+            device.log_admin_entry(
+                None, "%s → %s (Invalid device configuration)" % (
+                    prev_health_display, device.get_health_display()))
+            device.save()
+            logger.debug(
+                "%s → %s (Invalid device configuration for %s)" % (
+                    prev_health_display, device.get_health_display(), device.hostname))
+            continue
+
         job_dict = yaml.safe_load(job.definition)
         if 'protocols' in job_dict and 'lava-vland' in job_dict['protocols']:
             if not match_vlan_interface(device, job_dict):
@@ -256,7 +276,7 @@ def transition_multinode_jobs(logger):
             # apply the complete list to all jobs in this group
             definition = yaml.safe_load(sub_job.definition)
             definition['protocols']['lava-multinode']['roles'] = devices
-            sub_job.definition = yaml.dump(definition)
+            sub_job.definition = yaml.safe_dump(definition)
             # transition the job and device
             sub_job.go_state_scheduled()
             sub_job.save()
