@@ -1,15 +1,27 @@
-from __future__ import unicode_literals
+# -*- coding: utf-8 -*-
+# Copyright (C) 2018 Linaro Limited
+#
+# Author: Neil Williams <neil.williams@linaro.org>
+#         Remi Duraffort <remi.duraffort@linaro.org>
+#
+# This file is part of LAVA.
+#
+# LAVA is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License version 3
+# as published by the Free Software Foundation
+#
+# LAVA is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU Affero General Public License
+# along with LAVA.  If not, see <http://www.gnu.org/licenses/>.
 
 import yaml
-import django
 import logging
 import random
-from django.contrib.admin.models import (
-    ADDITION,
-    CHANGE,
-    LogEntry,
-)
-from django.template import defaultfilters as filters
+from django.contrib.admin.models import LogEntry
 from django.utils.html import escape
 from django.utils.safestring import mark_safe
 from django.utils.timesince import timesince
@@ -21,7 +33,6 @@ from lava_scheduler_app.models import (
     DeviceType,
     Worker,
 )
-from lava_results_app.models import TestCase
 from lava.utils.lavatable import LavaTable
 from django.db.models import Q
 from django.utils import timezone
@@ -42,7 +53,7 @@ class IDLinkColumn(tables.Column):
 
     def __init__(self, verbose_name="ID", **kw):
         kw['verbose_name'] = verbose_name
-        super(IDLinkColumn, self).__init__(**kw)
+        super().__init__(**kw)
 
     def render(self, record, table=None):  # pylint: disable=arguments-differ,unused-argument
         return pklink(record)
@@ -63,7 +74,7 @@ def pklink(record):
     if isinstance(record, TestJob):
         if record.sub_jobs_list:
             job_id = record.sub_id
-    return mark_safe(
+    return mark_safe(  # nosec - internal data
         '<a href="%s" title="job summary">%s</a>' % (
             record.get_absolute_url(),
             escape(job_id)))
@@ -73,7 +84,7 @@ class ExpandedStatusColumn(tables.Column):
 
     def __init__(self, verbose_name="Expanded Status", **kw):
         kw['verbose_name'] = verbose_name
-        super(ExpandedStatusColumn, self).__init__(**kw)
+        super().__init__(**kw)
 
     def render(self, record):
         """
@@ -84,13 +95,13 @@ class ExpandedStatusColumn(tables.Column):
         logger = logging.getLogger('lava_scheduler_app')
         if record.state == Device.STATE_RUNNING:
             current_job = record.current_job()
-            return mark_safe("Running job #%s - %s submitted by %s" % (
+            return mark_safe("Running job #%s - %s submitted by %s" % (  # nosec - internal data
                 pklink(current_job),
                 current_job.description,
                 current_job.submitter))
         elif record.state == Device.STATE_RESERVED:
             current_job = record.current_job()
-            return mark_safe("Reserved for job #%s (%s) \"%s\" submitted by %s" % (
+            return mark_safe("Reserved for job #%s (%s) \"%s\" submitted by %s" % (  # nosec - internal data
                 pklink(current_job),
                 current_job.get_state_display(),
                 current_job.description,
@@ -105,7 +116,7 @@ class RestrictedDeviceColumn(tables.Column):
 
     def __init__(self, verbose_name="Submissions restricted to", **kw):
         kw['verbose_name'] = verbose_name
-        super(RestrictedDeviceColumn, self).__init__(**kw)
+        super().__init__(**kw)
 
     def render(self, record):
         """
@@ -115,8 +126,12 @@ class RestrictedDeviceColumn(tables.Column):
         :return: a text string describing the restrictions on this device.
         """
         label = None
-        if record.health in [Device.HEALTH_BAD, Device.HEALTH_MAINTENANCE, Device.HEALTH_RETIRED]:
-            return "no submissions possible."
+        if record.health == Device.HEALTH_BAD:
+            return "Health check failed: no test jobs will be scheduled."
+        if record.health == Device.HEALTH_MAINTENANCE:
+            return "No test jobs will be scheduled."
+        if record.health == Device.HEALTH_RETIRED:
+            return "Retired: no submissions possible."
         if record.is_public:
             return ""
         if record.user:
@@ -145,7 +160,7 @@ def all_jobs_with_custom_sort():
 class JobErrorsTable(LavaTable):
 
     def __init__(self, *args, **kwargs):
-        super(LavaTable, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.length = 10
 
     job = tables.Column(verbose_name="Job", empty_values=[""])
@@ -168,7 +183,7 @@ class JobErrorsTable(LavaTable):
         if record.suite.job.actual_device is None:
             return ""
         else:
-            return mark_safe(
+            return mark_safe(  # nosec - internal data
                 '<a href="%s" title="device details">%s</a>' % (
                     record.suite.job.actual_device.get_absolute_url(),
                     escape(record.suite.job.actual_device.hostname)))
@@ -180,7 +195,7 @@ class JobErrorsTable(LavaTable):
         return record.action_metadata["error_msg"]
 
     def render_job(self, record):
-        return mark_safe('<a href="%s">%s</a>' % (record.suite.job.get_absolute_url(), record.suite.job.pk))
+        return mark_safe('<a href="%s">%s</a>' % (record.suite.job.get_absolute_url(), record.suite.job.pk))  # nosec - internal data
 
     class Meta(LavaTable.Meta):
         model = TestCase
@@ -204,7 +219,7 @@ class JobTable(LavaTable):
     ensure those are copied into the new class.
     """
     def __init__(self, *args, **kwargs):
-        super(JobTable, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.length = 25
 
     id = tables.Column(verbose_name="ID")
@@ -216,11 +231,14 @@ class JobTable(LavaTable):
     duration.orderable = False
     submit_time = tables.DateColumn(format="Nd, g:ia")
     end_time = tables.DateColumn(format="Nd, g:ia")
+    state = tables.Column()
+    state.orderable = False
 
     def render_state(self, record):
         if record.state == TestJob.STATE_RUNNING:
-            return mark_safe('<span class="text-info"><strong>%s</strong></span>' %
-                             record.get_state_display())
+            return mark_safe(  # nosec - internal data
+                '<span class="text-info"><strong>%s</strong></span>' %
+                record.get_state_display())
         elif record.state == TestJob.STATE_FINISHED:
             if record.health == TestJob.HEALTH_UNKNOWN:
                 text = 'text-default'
@@ -230,11 +248,13 @@ class JobTable(LavaTable):
                 text = 'text-danger'
             elif record.health == TestJob.HEALTH_CANCELED:
                 text = 'text-warning'
-            return mark_safe('<span class="%s"><strong>%s</strong></span>' %
-                             (text, record.get_health_display()))
+            return mark_safe(  # nosec - internal data
+                '<span class="%s"><strong>%s</strong></span>' %
+                (text, record.get_health_display()))
         else:
-            return mark_safe('<span class="text-muted"><strong>%s</strong></span>' %
-                             record.get_state_display())
+            return mark_safe(  # nosec - internal data
+                '<span class="text-muted"><strong>%s</strong></span>' %
+                record.get_state_display())
 
     def render_device(self, record):
         if record.actual_device:
@@ -242,7 +262,7 @@ class JobTable(LavaTable):
             retval = pklink(record.actual_device)
         elif record.requested_device_type:
             device_type = record.requested_device_type
-            retval = mark_safe('<i>%s</i>' % escape(record.requested_device_type.pk))
+            retval = mark_safe('<i>%s</i>' % escape(record.requested_device_type.pk))  # nosec - internal data
         elif record.dynamic_connection:
             return 'connection'
         else:
@@ -318,7 +338,7 @@ class IndexJobTable(JobTable):
     end_time = tables.DateColumn("Nd, g:ia")
 
     def __init__(self, *args, **kwargs):
-        super(IndexJobTable, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.length = 25
 
     class Meta(JobTable.Meta):  # pylint: disable=too-few-public-methods,no-init,no-self-use
@@ -336,14 +356,14 @@ class IndexJobTable(JobTable):
 class TagsColumn(tables.Column):
 
     def render(self, value):
-        tag_id = 'tag-%s' % "".join(random.choice("abcdefghijklmnopqrstuvwxyz") for _ in range(8))
+        tag_id = 'tag-%s' % "".join(random.choice("abcdefghijklmnopqrstuvwxyz") for _ in range(8))  # nosec - not crypto
         tags = ''
         values = list(value.all())
         if len(values) > 0:
             tags = '<p class="collapse" id="%s">' % tag_id
             tags += ',<br>'.join('<abbr data-toggle="tooltip" title="%s">%s</abbr>' % (tag.description, tag.name) for tag in values)
             tags += '</p><a class="btn btn-xs btn-success" data-toggle="collapse" data-target="#%s"><span class="glyphicon glyphicon-eye-open"></span></a>' % tag_id
-        return mark_safe(tags)
+        return mark_safe(tags)  # nosec - internal data
 
 
 class FailedJobTable(JobTable):
@@ -361,7 +381,7 @@ class FailedJobTable(JobTable):
     end_time = tables.DateColumn("Nd, g:ia")
 
     def __init__(self, *args, **kwargs):
-        super(FailedJobTable, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.length = 10
 
     def render_failure_comment(self, record):
@@ -411,7 +431,7 @@ class LongestJobTable(JobTable):
     running.orderable = False
 
     def __init__(self, *args, **kwargs):
-        super(LongestJobTable, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.length = 10
 
     def render_running(self, record):  # pylint: disable=no-self-use
@@ -448,7 +468,7 @@ class OverviewJobsTable(JobTable):
     end_time = tables.DateColumn("Nd, g:ia")
 
     def __init__(self, *args, **kwargs):
-        super(OverviewJobsTable, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.length = 10
 
     class Meta(JobTable.Meta):  # pylint: disable=too-few-public-methods,no-init,no-self-use
@@ -476,7 +496,7 @@ class RecentJobsTable(JobTable):
     end_time = tables.DateColumn("Nd, g:ia")
 
     def __init__(self, *args, **kwargs):
-        super(RecentJobsTable, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.length = 10
 
     class Meta(JobTable.Meta):  # pylint: disable=too-few-public-methods,no-init,no-self-use
@@ -496,7 +516,7 @@ class RecentJobsTable(JobTable):
 class DeviceHealthTable(LavaTable):
 
     def __init__(self, *args, **kwargs):
-        super(DeviceHealthTable, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.length = 25
 
     def render_last_health_report_job(self, record):  # pylint: disable=no-self-use
@@ -534,7 +554,7 @@ class DeviceHealthTable(LavaTable):
 class DeviceTypeTable(LavaTable):
 
     def __init__(self, *args, **kwargs):
-        super(DeviceTypeTable, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.length = 50
 
     def render_idle(self, record):  # pylint: disable=no-self-use
@@ -575,15 +595,12 @@ class DeviceTypeTable(LavaTable):
             'architecture', 'health_denominator', 'health_frequency',
             'processor', 'cpu_model', 'bits', 'cores', 'core_count', 'description'
         ]
-        searches = {
-            'name': 'contains',
-        }
 
 
 class DeviceTable(LavaTable):
 
     def __init__(self, *args, **kwargs):
-        super(DeviceTable, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.length = 50
 
     def render_device_type(self, record):  # pylint: disable=no-self-use
@@ -591,15 +608,15 @@ class DeviceTable(LavaTable):
 
     def render_health(self, record):
         if record.health == Device.HEALTH_GOOD:
-            return mark_safe('<strong class="text-success">Good</strong>')
+            return mark_safe('<strong class="text-success">Good</strong>')  # nosec - internal data
         elif record.health in [Device.HEALTH_UNKNOWN, Device.HEALTH_LOOPING]:
-            return mark_safe('<span class="text-info">%s</span>' % record.get_health_display())
+            return mark_safe('<span class="text-info">%s</span>' % record.get_health_display())  # nosec - internal data
         elif record.health == Device.HEALTH_BAD:
-            return mark_safe('<span class="text-danger">Bad</span>')
+            return mark_safe('<span class="text-danger">Bad</span>')  # nosec - internal data
         elif record.health == Device.HEALTH_MAINTENANCE:
-            return mark_safe('<span class="text-warning">Maintenance</span>')
+            return mark_safe('<span class="text-warning">Maintenance</span>')  # nosec - internal data
         else:
-            return mark_safe('<span class="text-muted">Retired</span>')
+            return mark_safe('<span class="text-muted">Retired</span>')  # nosec - internal data
 
     hostname = tables.TemplateColumn('''
     <a href="{{ record.get_absolute_url }}">{{ record.hostname }}</a>
@@ -640,7 +657,7 @@ class DeviceTable(LavaTable):
 class WorkerTable(tables.Table):  # pylint: disable=too-few-public-methods,no-init
 
     def __init__(self, *args, **kwargs):
-        super(WorkerTable, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.length = 10
         self.show_help = True
 
@@ -650,19 +667,19 @@ class WorkerTable(tables.Table):  # pylint: disable=too-few-public-methods,no-in
 
     def render_state(self, record):
         if record.state == Worker.STATE_ONLINE:
-            return mark_safe('<span class="glyphicon glyphicon-ok text-success"></span> %s' % record.get_state_display())
+            return mark_safe('<span class="glyphicon glyphicon-ok text-success"></span> %s' % record.get_state_display())  # nosec - internal data
         elif record.health == Worker.HEALTH_ACTIVE:
-            return mark_safe('<span class="glyphicon glyphicon-fire text-danger"></span> %s' % record.get_state_display())
+            return mark_safe('<span class="glyphicon glyphicon-fire text-danger"></span> %s' % record.get_state_display())  # nosec - internal data
         else:
-            return mark_safe('<span class="glyphicon glyphicon-remove text-danger"></span> %s' % record.get_state_display())
+            return mark_safe('<span class="glyphicon glyphicon-remove text-danger"></span> %s' % record.get_state_display())  # nosec - internal data
 
     def render_health(self, record):
         if record.health == Worker.HEALTH_ACTIVE:
-            return mark_safe('<span class="glyphicon glyphicon-ok text-success"></span> %s' % record.get_health_display())
+            return mark_safe('<span class="glyphicon glyphicon-ok text-success"></span> %s' % record.get_health_display())  # nosec - internal data
         elif record.health == Worker.HEALTH_MAINTENANCE:
-            return mark_safe('<span class="glyphicon glyphicon-wrench text-warning"></span> %s' % record.get_health_display())
+            return mark_safe('<span class="glyphicon glyphicon-wrench text-warning"></span> %s' % record.get_health_display())  # nosec - internal data
         else:
-            return mark_safe('<span class="glyphicon glyphicon-remove text-danger"></span> %s' % record.get_health_display())
+            return mark_safe('<span class="glyphicon glyphicon-remove text-danger"></span> %s' % record.get_health_display())  # nosec - internal data
 
     def render_last_ping(self, record):
         return timesince(record.last_ping)
@@ -677,7 +694,7 @@ class WorkerTable(tables.Table):  # pylint: disable=too-few-public-methods,no-in
 class LogEntryTable(tables.Table):
 
     def __init__(self, *args, **kwargs):
-        super(LogEntryTable, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.length = 10
 
     action_time = tables.DateColumn(format="Nd, g:ia")
@@ -686,16 +703,13 @@ class LogEntryTable(tables.Table):
     change_message.orderable = False
 
     def render_change_message(self, record):
-        if django.VERSION > (1, 10):
-            message = record.get_change_message()
-        else:
-            message = record.change_message
+        message = record.get_change_message()
         if record.is_change():
             return message
         elif record.is_addition():
-            return mark_safe('<span class="glyphicon glyphicon-plus text-success"></span> %s' % message)
+            return mark_safe('<span class="glyphicon glyphicon-plus text-success"></span> %s' % message)  # nosec - internal data
         else:
-            return mark_safe('<span class="glyphicon glyphicon-remove text-danger"></span> %s' % message)
+            return mark_safe('<span class="glyphicon glyphicon-remove text-danger"></span> %s' % message)  # nosec - internal data
 
     class Meta(LavaTable.Meta):
         model = LogEntry
@@ -763,7 +777,7 @@ class QueueJobsTable(JobTable):
     end_time = tables.DateColumn("Nd, g:ia")
 
     def __init__(self, *args, **kwargs):
-        super(QueueJobsTable, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.length = 50
 
     class Meta(JobTable.Meta):  # pylint: disable=too-few-public-methods,no-init,no-self-use
@@ -781,7 +795,7 @@ class QueueJobsTable(JobTable):
 class PassingHealthTable(DeviceHealthTable):
 
     def __init__(self, *args, **kwargs):
-        super(PassingHealthTable, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.length = 25
 
     def render_device_type(self, record):  # pylint: disable=no-self-use
@@ -789,7 +803,7 @@ class PassingHealthTable(DeviceHealthTable):
 
     def render_last_health_report_job(self, record):  # pylint: disable=no-self-use
         report = record.last_health_report_job
-        return mark_safe('<a href="%s">%s</a>' % (report.get_absolute_url(), report))
+        return mark_safe('<a href="%s">%s</a>' % (report.get_absolute_url(), report))  # nosec - internal data
 
     device_type = tables.Column()
 
@@ -818,7 +832,7 @@ class RunningTable(LavaTable):
     """
 
     def __init__(self, *args, **kwargs):
-        super(RunningTable, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.length = 50
 
     # deprecated: dynamic connections are TestJob without a device
