@@ -36,10 +36,7 @@ class TestFastbootTemplates(BaseTemplate.BaseTemplateCases):
 {% set static_info = [{'board_id': 'R32D300FRYP'}] %}
 """
         self.assertTrue(self.validate_data('nexus4-01', data))
-        test_template = prepare_jinja_template('nexus4-01', data)
-        rendered = test_template.render()
-        template_dict = yaml.load(rendered)
-        self.assertEqual('nexus4', template_dict['device_type'])
+        template_dict = prepare_jinja_template('nexus4-01', data, raw=False)
         self.assertEqual('R32D300FRYP', template_dict['adb_serial_number'])
         self.assertEqual('R32D300FRYP', template_dict['fastboot_serial_number'])
         self.assertEqual([], template_dict['fastboot_options'])
@@ -49,20 +46,22 @@ class TestFastbootTemplates(BaseTemplate.BaseTemplateCases):
 
     def test_x15_template(self):
         data = """{% extends 'x15.jinja2' %}
-{% set adb_serial_number = 'R32D300FRYP' %}
-{% set fastboot_serial_number = 'R32D300FRYP' %}
+{% set adb_serial_number = '1234567890' %}
+{% set fastboot_serial_number = '1234567890' %}
+{% set interrupt_prompt = "interrupt bootloader" %}
 """
         self.assertTrue(self.validate_data('x15-01', data))
-        test_template = prepare_jinja_template('x15-01', data)
-        rendered = test_template.render()
-        template_dict = yaml.load(rendered)
-        self.assertEqual('x15', template_dict['device_type'])
-        self.assertEqual('R32D300FRYP', template_dict['adb_serial_number'])
-        self.assertEqual('R32D300FRYP', template_dict['fastboot_serial_number'])
+        template_dict = prepare_jinja_template('x15-01', data, raw=False)
+        self.assertIsInstance(template_dict['adb_serial_number'], str)
+        self.assertEqual('1234567890', template_dict['adb_serial_number'])
+        self.assertEqual('1234567890', template_dict['fastboot_serial_number'])
         self.assertEqual([], template_dict['fastboot_options'])
         self.assertIn('u-boot', template_dict['actions']['boot']['methods'])
         self.assertIn('parameters', template_dict['actions']['boot']['methods']['u-boot'])
         self.assertIn('interrupt_prompt', template_dict['actions']['boot']['methods']['u-boot']['parameters'])
+        self.assertEqual(
+            'interrupt bootloader',
+            template_dict['actions']['boot']['methods']['u-boot']['parameters']['interrupt_prompt'])
         # fastboot deploy to eMMC
         self.assertIn('mmc', template_dict['actions']['boot']['methods']['u-boot'])
         self.assertIn('commands', template_dict['actions']['boot']['methods']['u-boot']['mmc'])
@@ -74,6 +73,8 @@ class TestFastbootTemplates(BaseTemplate.BaseTemplateCases):
                 # x15 needs both consoles enabled.
                 self.assertIn('ttyS2', command)
                 self.assertNotIn('console=ttyO2', command)
+        for board in template_dict['device_info']:
+            self.assertEqual(template_dict['fastboot_serial_number'], board['board_id'])
 
     def test_sharkl2_template(self):
         data = """{% extends 'sharkl2.jinja2' %}
@@ -81,10 +82,7 @@ class TestFastbootTemplates(BaseTemplate.BaseTemplateCases):
 {% set fastboot_serial_number = 'R32D300FRYP' %}
 """
         self.assertTrue(self.validate_data('sharkl2-01', data))
-        test_template = prepare_jinja_template('sharkl2-01', data)
-        rendered = test_template.render()
-        template_dict = yaml.load(rendered)
-        self.assertEqual('sharkl2', template_dict['device_type'])
+        template_dict = prepare_jinja_template('sharkl2-01', data, raw=False)
         self.assertEqual('R32D300FRYP', template_dict['adb_serial_number'])
         self.assertEqual('R32D300FRYP', template_dict['fastboot_serial_number'])
         self.assertEqual([], template_dict['fastboot_options'])
@@ -105,10 +103,8 @@ class TestFastbootTemplates(BaseTemplate.BaseTemplateCases):
 {% set static_info = [{'board_id': 'R32D300FRYP'}] %}
 """
         self.assertTrue(self.validate_data('staging-nexus10-01', data))
-        test_template = prepare_jinja_template(
-            'staging-pixel-01', data)
-        rendered = test_template.render()
-        template_dict = yaml.load(rendered)
+        template_dict = prepare_jinja_template(
+            'staging-pixel-01', data, raw=False)
         self.assertIsNotNone(template_dict)
         self.assertIsInstance(template_dict['device_info'], list)
         self.assertIsInstance(template_dict['static_info'], list)
@@ -118,9 +114,7 @@ class TestFastbootTemplates(BaseTemplate.BaseTemplateCases):
             data = hikey.read()
         self.assertIsNotNone(data)
         self.assertTrue(self.validate_data('hi6220-hikey-01', data))
-        test_template = prepare_jinja_template('staging-hikey-01', data)
-        rendered = test_template.render()
-        template_dict = yaml.load(rendered)
+        template_dict = prepare_jinja_template('staging-hikey-01', data, raw=False)
         self.assertIsNotNone(template_dict)
         self.assertIsInstance(template_dict['device_info'], list)
         self.assertIsInstance(template_dict['static_info'], list)
@@ -140,9 +134,7 @@ class TestFastbootTemplates(BaseTemplate.BaseTemplateCases):
             data = hikey.read()
         self.assertIsNotNone(data)
         self.assertTrue(self.validate_data('hi6220-hikey-01', data))
-        test_template = prepare_jinja_template('staging-hikey-01', data)
-        rendered = test_template.render()
-        template_dict = yaml.load(rendered)
+        template_dict = prepare_jinja_template('staging-hikey-01', data, raw=False)
         self.assertIsNotNone(template_dict)
         self.assertIsInstance(template_dict['device_info'], list)
         self.assertIsInstance(template_dict['static_info'], list)
@@ -162,9 +154,7 @@ class TestFastbootTemplates(BaseTemplate.BaseTemplateCases):
         # test support for retreiving MAC from device.
         data += "{% set device_mac = '00:E0:4C:53:44:58' %}"
         self.assertTrue(self.validate_data('hi6220-hikey-01', data))
-        test_template = prepare_jinja_template('staging-hikey-01', data)
-        rendered = test_template.render()
-        template_dict = yaml.load(rendered)
+        template_dict = prepare_jinja_template('staging-hikey-01', data, raw=False)
         self.assertIn('parameters', template_dict)
         self.assertIn('interfaces', template_dict['parameters'])
         self.assertIn('target', template_dict['parameters']['interfaces'])
@@ -183,9 +173,7 @@ class TestFastbootTemplates(BaseTemplate.BaseTemplateCases):
             'devicetree': 'hi6220-hikey.dtb'
         }
         self.assertTrue(self.validate_data('hi6220-hikey-01', data))
-        test_template = prepare_jinja_template('staging-hikey-01', data)
-        rendered = test_template.render(**job_ctx)
-        template_dict = yaml.load(rendered)
+        template_dict = prepare_jinja_template('staging-hikey-01', data, job_ctx=job_ctx, raw=False)
         self.assertIsNotNone(template_dict)
         self.assertIsNotNone(template_dict['actions']['boot']['methods'])
         self.assertIn('grub-efi', template_dict['actions']['boot']['methods'])
@@ -218,9 +206,7 @@ class TestFastbootTemplates(BaseTemplate.BaseTemplateCases):
         self.assertIsNotNone(data)
         job_ctx = {}
         self.assertTrue(self.validate_data('hi6220-hikey-01', data))
-        test_template = prepare_jinja_template('staging-hikey-01', data)
-        rendered = test_template.render(**job_ctx)
-        template_dict = yaml.load(rendered)
+        template_dict = prepare_jinja_template('staging-hikey-01', data, job_ctx=job_ctx, raw=False)
         validate_device(template_dict)
         self.assertIsNotNone(template_dict)
         self.assertIn('commands', template_dict)
@@ -247,9 +233,7 @@ class TestFastbootTemplates(BaseTemplate.BaseTemplateCases):
             'devicetree': 'hi960-hikey.dtb'
         }
         # self.assertTrue(self.validate_data('hi960-hikey-01', data))
-        test_template = prepare_jinja_template('staging-hikey-01', data)
-        rendered = test_template.render(**job_ctx)
-        template_dict = yaml.load(rendered)
+        template_dict = prepare_jinja_template('staging-hikey-01', data, job_ctx=job_ctx, raw=False)
         self.assertIsNotNone(template_dict)
         self.assertIsNotNone(template_dict['actions']['boot']['methods'])
         self.assertNotIn('menu_options', template_dict['actions']['boot']['methods']['grub'])
@@ -268,9 +252,7 @@ class TestFastbootTemplates(BaseTemplate.BaseTemplateCases):
         # test support for retreiving MAC from device using base-fastboot.
         data += "{% set device_mac = '00:E0:4C:53:44:58' %}"
         self.assertTrue(self.validate_data('hi6220-hikey-01', data))
-        test_template = prepare_jinja_template('staging-hikey-01', data)
-        rendered = test_template.render()
-        template_dict = yaml.load(rendered)
+        template_dict = prepare_jinja_template('staging-hikey-01', data, job_ctx=None, raw=False)
         self.assertIn('parameters', template_dict)
         self.assertIn('interfaces', template_dict['parameters'])
         self.assertIn('target', template_dict['parameters']['interfaces'])
@@ -289,10 +271,8 @@ class TestFastbootTemplates(BaseTemplate.BaseTemplateCases):
 {% set static_info = [{'board_id': '10de1214adae123'}] %}
 """
         self.assertTrue(self.validate_data('staging-nexus5x-01', data))
-        test_template = prepare_jinja_template(
-            'staging-pixel-01', data)
-        rendered = test_template.render()
-        template_dict = yaml.load(rendered)
+        template_dict = prepare_jinja_template(
+            'staging-pixel-01', data, raw=False)
         self.assertIsNotNone(template_dict)
         self.assertIsInstance(template_dict['device_info'], list)
         self.assertIsInstance(template_dict['static_info'], list)
@@ -305,10 +285,8 @@ class TestFastbootTemplates(BaseTemplate.BaseTemplateCases):
 {% set static_info = [{'board_id': 'FDAC1231DAD'}] %}
 """
         self.assertTrue(self.validate_data('staging-pixel-01', data))
-        test_template = prepare_jinja_template(
-            'staging-pixel-01', data)
-        rendered = test_template.render()
-        template_dict = yaml.load(rendered)
+        template_dict = prepare_jinja_template(
+            'staging-pixel-01', data, raw=False)
         self.assertIsNotNone(template_dict)
         self.assertIsInstance(template_dict['device_info'], list)
         self.assertIsInstance(template_dict['static_info'], list)
@@ -324,11 +302,8 @@ class TestFastbootTemplates(BaseTemplate.BaseTemplateCases):
 {% set fastboot_serial_number = 'e080c212' %}
         """
         self.assertTrue(self.validate_data('staging-ifc6410-01', data))
-        test_template = prepare_jinja_template('staging-ifc6410-01',
-                                               data)
-        rendered = test_template.render()
-        template_dict = yaml.load(rendered)
-        self.assertEqual('ifc6410', template_dict['device_type'])
+        template_dict = prepare_jinja_template(
+            'staging-ifc6410-01', data, raw=False)
         self.assertEqual('e080c212', template_dict['adb_serial_number'])
         self.assertEqual('e080c212', template_dict['fastboot_serial_number'])
         self.assertEqual([], template_dict['fastboot_options'])
@@ -344,10 +319,7 @@ class TestFastbootTemplates(BaseTemplate.BaseTemplateCases):
 {% set fastboot_serial_number = '3083f595' %}
 """
         self.assertTrue(self.validate_data('db820c-01', data))
-        test_template = prepare_jinja_template('db820c-01', data)
-        rendered = test_template.render()
-        template_dict = yaml.load(rendered)
-        self.assertEqual('dragonboard-820c', template_dict['device_type'])
+        template_dict = prepare_jinja_template('db820c-01', data, raw=False)
         self.assertEqual('3083f595', template_dict['adb_serial_number'])
         self.assertEqual('3083f595', template_dict['fastboot_serial_number'])
         self.assertEqual([], template_dict['fastboot_options'])
@@ -356,9 +328,7 @@ class TestFastbootTemplates(BaseTemplate.BaseTemplateCases):
         with open(os.path.join(os.path.dirname(__file__), 'devices', 'hi6220-hikey-bl-01.jinja2')) as hikey:
             data = hikey.read()
         self.assertTrue(self.validate_data('hi620-bl-01', data))
-        test_template = prepare_jinja_template('hi620-bl-01', data)
-        rendered = test_template.render()
-        template_dict = yaml.load(rendered)
+        template_dict = prepare_jinja_template('hi620-bl-01', data, raw=False)
         recovery = template_dict['actions']['deploy']['methods']
         self.assertIsNotNone('recovery', recovery)
         self.assertIn('recovery', recovery)
@@ -374,3 +344,33 @@ class TestFastbootTemplates(BaseTemplate.BaseTemplateCases):
             ['/home/neil/lava-lab/shared/lab-scripts/eth008_control -a 10.15.0.171 -r 1 -s on',
              '/home/neil/lava-lab/shared/lab-scripts/eth008_control -a 10.15.0.171 -r 2 -s off'],
             recovery['recovery']['commands']['recovery_exit'])
+
+    def test_sdm845_mtp_template(self):
+        fastboot_cmd_order = ['update',
+                              'ptable',
+                              'partition',
+                              'hyp',
+                              'modem',
+                              'rpm',
+                              'sbl1',
+                              'sbl2',
+                              'sec',
+                              'tz',
+                              'aboot',
+                              'cdt',
+                              'boot',
+                              'rootfs',
+                              'vendor',
+                              'system',
+                              'cache',
+                              'userdata']
+
+        rendered = self.render_device_dictionary_file('sdm845-mtp-05.jinja2')
+        template_dict = yaml.safe_load(rendered)
+        self.assertEqual('5c302cef', template_dict['adb_serial_number'])
+        self.assertEqual('5c302cef', template_dict['fastboot_serial_number'])
+        self.assertEqual([], template_dict['fastboot_options'])
+
+        for cmd in template_dict['flash_cmds_order']:
+            idx = template_dict['flash_cmds_order'].index(cmd)
+            self.assertEqual(cmd, fastboot_cmd_order[idx])
