@@ -4,6 +4,8 @@ import yaml
 import jinja2
 import unittest
 import logging
+import subprocess  # nosec unit test support
+from nose.tools import nottest
 from lava_scheduler_app.models import (
     Device,
     DeviceType,
@@ -11,10 +13,8 @@ from lava_scheduler_app.models import (
     Tag,
     DevicesUnavailableException,
     _pipeline_protocols,
-    _check_submit_to_device,
 )
 from lava_scheduler_app.dbutils import match_vlan_interface
-from django.db.models import Q
 from django.contrib.auth.models import Group, Permission, User
 from lava_scheduler_app.utils import (
     split_multinode_yaml,
@@ -23,7 +23,6 @@ from lava_scheduler_app.tests.test_submission import ModelFactory, TestCaseWithF
 from lava_scheduler_app.schema import (
     validate_submission,
     validate_device,
-    include_yaml,
     SubmissionException
 )
 from lava_dispatcher.device import PipelineDevice
@@ -34,7 +33,6 @@ from lava_dispatcher.actions.boot.qemu import BootQEMU
 from lava_dispatcher.protocols.multinode import MultinodeProtocol
 from lava_common.constants import SYS_CLASS_KVM
 from lava_dispatcher.test.utils import DummyLogger
-from django_restricted_resource.managers import RestrictedResourceQuerySet
 from unittest import TestCase
 
 
@@ -52,7 +50,7 @@ class YamlFactory(ModelFactory):
     """
 
     def __init__(self):
-        super(YamlFactory, self).__init__()
+        super().__init__()
         Device.CONFIG_PATH = os.path.join(os.getcwd(), 'lava_scheduler_app', 'tests', 'devices')
         logging.basicConfig(stream=sys.stdout, level=logging.DEBUG)
         logger = logging.getLogger('unittests')
@@ -89,7 +87,7 @@ class YamlFactory(ModelFactory):
     def make_job_data(self, actions=None, **kw):
         sample_job_file = os.path.join(os.path.dirname(__file__), 'devices', 'qemu.yaml')
         with open(sample_job_file, 'r') as test_support:
-            data = yaml.load(test_support)
+            data = yaml.safe_load(test_support)
         data.update(kw)
         return data
 
@@ -103,7 +101,7 @@ class PipelineDeviceTags(TestCaseWithFactory):
     Same tests as test_submission but converted to use and look for YAML.
     """
     def setUp(self):
-        super(PipelineDeviceTags, self).setUp()
+        super().setUp()
         self.factory = YamlFactory()
         self.device_type = self.factory.make_device_type()
         self.conf = {
@@ -114,7 +112,7 @@ class PipelineDeviceTags(TestCaseWithFactory):
         }
 
     def test_make_job_yaml(self):
-        data = yaml.load(self.factory.make_job_yaml())
+        data = yaml.safe_load(self.factory.make_job_yaml())
         self.assertIn('device_type', data)
         self.assertNotIn('timeout', data)
         self.assertIn('timeouts', data)
@@ -149,7 +147,7 @@ class PipelineDeviceTags(TestCaseWithFactory):
             self.factory.ensure_tag('usb'),
             self.factory.ensure_tag('sata')
         ]
-        data = yaml.load(self.factory.make_job_yaml(tags=['usb', 'sata']))
+        data = yaml.safe_load(self.factory.make_job_yaml(tags=['usb', 'sata']))
         validate_submission(data)
         device = self.factory.make_device(self.device_type, 'fakeqemu4', tags=tag_list)
         job_ctx = data.get('context', {})
@@ -235,7 +233,7 @@ class PipelineDeviceTags(TestCaseWithFactory):
 class TestPipelineSubmit(TestCaseWithFactory):
 
     def setUp(self):
-        super(TestPipelineSubmit, self).setUp()
+        super().setUp()
         self.factory = YamlFactory()
         self.device_type = self.factory.make_device_type()
         self.factory.make_device(device_type=self.device_type, hostname="fakeqemu1")
@@ -265,7 +263,7 @@ class TestPipelineSubmit(TestCaseWithFactory):
         user = self.factory.make_user()
         job = TestJob.from_yaml_and_user(
             self.factory.make_job_yaml(), user)
-        job_def = yaml.load(job.definition)
+        job_def = yaml.safe_load(job.definition)
         job_ctx = job_def.get('context', {})
         device_config = device.load_configuration(job_ctx)  # raw dict
         self.assertTrue(device.is_valid(system=False))
@@ -274,7 +272,7 @@ class TestPipelineSubmit(TestCaseWithFactory):
             'qemu-system-x86_64'
         )
 
-        job_data = yaml.load(self.factory.make_job_yaml())
+        job_data = yaml.safe_load(self.factory.make_job_yaml())
         job_data['context'].update({'netdevice': 'tap'})
         job_ctx = job_data.get('context', {})
         device_config = device.load_configuration(job_ctx)  # raw dict
@@ -324,7 +322,7 @@ class TestPipelineSubmit(TestCaseWithFactory):
         )
 
     def test_metadata(self):
-        data = yaml.load(self.factory.make_job_yaml(metadata={15: 'test'}))
+        data = yaml.safe_load(self.factory.make_job_yaml(metadata={15: 'test'}))
         validate_submission(data)
         metadata = {
             'build-url': None,
@@ -334,13 +332,13 @@ class TestPipelineSubmit(TestCaseWithFactory):
             'git-url': 'https://git.linaro.org/zephyrproject-org/zephyr.git',
             'git-commit': 1234
         }
-        data = yaml.load(self.factory.make_job_yaml(metadata=metadata))
+        data = yaml.safe_load(self.factory.make_job_yaml(metadata=metadata))
         self.assertRaises(SubmissionException, validate_submission, data)
         metadata['build-url'] = 'http://nowhere.com'
-        data = yaml.load(self.factory.make_job_yaml(metadata=metadata))
+        data = yaml.safe_load(self.factory.make_job_yaml(metadata=metadata))
         self.assertRaises(SubmissionException, validate_submission, data)
         metadata['zephyr-gcc-variant'] = '4.9'
-        data = yaml.load(self.factory.make_job_yaml(metadata=metadata))
+        data = yaml.safe_load(self.factory.make_job_yaml(metadata=metadata))
         validate_submission(data)
 
     def test_command_list(self):
@@ -370,7 +368,7 @@ class TestPipelineSubmit(TestCaseWithFactory):
         self.assertIn('timeouts', device_dict)
 
     def test_auto_login(self):
-        data = yaml.load(self.factory.make_job_yaml())
+        data = yaml.safe_load(self.factory.make_job_yaml())
         validate_submission(data)
 
         boot_params = None
@@ -405,7 +403,7 @@ class TestPipelineSubmit(TestCaseWithFactory):
 
         # public set in the YAML
         yaml_str = self.factory.make_job_yaml()
-        yaml_data = yaml.load(yaml_str)
+        yaml_data = yaml.safe_load(yaml_str)
         job = TestJob.from_yaml_and_user(
             yaml_str, user)
         self.assertTrue(job.is_public)
@@ -435,7 +433,7 @@ class TestPipelineSubmit(TestCaseWithFactory):
         self.assertFalse(job2.can_view(user))
         self.assertTrue(job2.can_view(user3))
 
-        job_data = yaml.load(self.factory.make_job_yaml())
+        job_data = yaml.safe_load(self.factory.make_job_yaml())
         job_data['visibility'] = {'group': [group1.name]}
         param = job_data['visibility']
         if isinstance(param, dict):
@@ -457,7 +455,7 @@ class TestPipelineSubmit(TestCaseWithFactory):
         user = self.factory.make_user()
         # public set in the YAML
         yaml_str = self.factory.make_job_yaml()
-        yaml_data = yaml.load(yaml_str)
+        yaml_data = yaml.safe_load(yaml_str)
         job = TestJob.from_yaml_and_user(
             yaml_str, user)
         self.assertTrue(job.is_public)
@@ -466,17 +464,17 @@ class TestPipelineSubmit(TestCaseWithFactory):
         self.assertEqual(job.pipeline_compatibility, 0)
         self.assertNotIn('compatibility', yaml_data)
         # FIXME: dispatcher master needs to make this kind of test more accessible.
-        definition = yaml.load(job.definition)
+        definition = yaml.safe_load(job.definition)
         self.assertNotIn('protocols', definition)
         job.actual_device = Device.objects.get(hostname='fakeqemu1')
-        job_def = yaml.load(job.definition)
+        job_def = yaml.safe_load(job.definition)
         job_ctx = job_def.get('context', {})
         parser = JobParser()
         device = job.actual_device
 
         try:
             device_config = device.load_configuration(job_ctx)  # raw dict
-        except (jinja2.TemplateError, yaml.YAMLError, IOError) as exc:
+        except (jinja2.TemplateError, yaml.YAMLError, OSError) as exc:
             # FIXME: report the exceptions as useful user messages
             self.fail("[%d] jinja2 error: %s" % (job.id, exc))
         if not device_config or not isinstance(device_config, dict):
@@ -509,46 +507,11 @@ class TestPipelineSubmit(TestCaseWithFactory):
         job.save()
         device.save()
 
-    def test_include_yaml_non_dict(self):
-        include_data = ['value1', 'value2']
-        self.assertRaises(
-            SubmissionException,
-            include_yaml,
-            self.factory.make_job_data(),
-            include_data)
-
-    def test_include_yaml_overwrite(self):
-        # Test overwrite of values.
-        job_data = self.factory.make_job_data()
-        include_data = {'priority': 'high'}
-        job_data = include_yaml(job_data, include_data)
-        self.assertEqual(job_data['priority'], 'high')
-
-        # Test in-depth overwrite.
-        include_data = {'timeouts': {'action': {'minutes': 10}}}
-        job_data = include_yaml(job_data, include_data)
-        self.assertEqual(job_data['timeouts']['action']['minutes'], 10)
-        self.assertEqual(job_data['timeouts']['job']['minutes'], 15)
-
-    def test_include_yaml_list_append(self):
-        job_data = self.factory.make_job_data()
-        include_data = {'actions': [
-            {'deploy': {'to': 'tmpfs', 'compression': 'gz', 'images': {}}}]}
-        job_data = include_yaml(job_data, include_data)
-        self.assertEqual(len(job_data['actions']), 4)
-        self.assertEqual(job_data['actions'][3], include_data['actions'][0])
-
-    def test_include_yaml(self):
-        job_data = self.factory.make_job_data()
-        include_data = {'key': 'value'}
-        job_data = include_yaml(job_data, include_data)
-        self.assertEqual(job_data['key'], 'value')
-
 
 class TestExtendsSubmit(TestCaseWithFactory):
 
     def setUp(self):
-        super(TestExtendsSubmit, self).setUp()
+        super().setUp()
         Device.HEALTH_CHECK_PATH = os.path.join(os.getcwd(), 'lava_scheduler_app', 'tests', 'health-checks')
         self.factory = YamlFactory()
         self.device_type = self.factory.make_device_type(name='juno-r2')
@@ -576,10 +539,24 @@ class TestExtendsSubmit(TestCaseWithFactory):
         self.assertIsNotNone(device1.get_health_check())
 
 
+@nottest
+def check_rpcinfo(server='127.0.0.1'):
+    """
+    Supports the unittest.SkipIf
+    needs to return True if skip, so
+    returns True on failure.
+    """
+    try:
+        subprocess.check_output(['/usr/sbin/rpcinfo', '-u', server, 'nfs', '3'])  # nosec unit test
+    except (OSError, subprocess.CalledProcessError):
+        return True
+    return False
+
+
 class TestYamlMultinode(TestCaseWithFactory):
 
     def setUp(self):
-        super(TestYamlMultinode, self).setUp()
+        super().setUp()
         self.factory = YamlFactory()
         logging.basicConfig(stream=sys.stdout, level=logging.DEBUG)
         logger = logging.getLogger('unittests')
@@ -591,7 +568,7 @@ class TestYamlMultinode(TestCaseWithFactory):
         logger.propagate = False
 
     def tearDown(self):
-        super(TestYamlMultinode, self).tearDown()
+        super().tearDown()
         Device.objects.all().delete()
 
     def test_multinode_split(self):
@@ -603,7 +580,7 @@ class TestYamlMultinode(TestCaseWithFactory):
         """
         server_check = os.path.join(os.path.dirname(__file__), 'sample_jobs', 'kvm-multinode-server.yaml')
         client_check = os.path.join(os.path.dirname(__file__), 'sample_jobs', 'kvm-multinode-client.yaml')
-        submission = yaml.load(open(
+        submission = yaml.safe_load(open(
             os.path.join(os.path.dirname(__file__), 'sample_jobs', 'kvm-multinode.yaml'), 'r'))
         target_group = 'arbitrary-group-id'  # for unit tests only
 
@@ -614,15 +591,16 @@ class TestYamlMultinode(TestCaseWithFactory):
                 del(job['protocols']['lava-multinode']['sub_id'])
                 yaml.dump(job)  # ensure the jobs can be serialised as YAML
                 if role == 'client':
-                    self.assertEqual(job, yaml.load(open(client_check, 'r')))
+                    self.assertEqual(job, yaml.safe_load(open(client_check, 'r')))
                 if role == 'server':
-                    self.assertEqual(job, yaml.load(open(server_check, 'r')))
+                    self.assertEqual(job, yaml.safe_load(open(server_check, 'r')))
 
+    @unittest.skipIf(check_rpcinfo(), "rpcinfo returns non-zero for nfs")
     def test_secondary_connection(self):
         user = self.factory.make_user()
         device_type = self.factory.make_device_type(name='mustang')
         device = self.factory.make_device(device_type, 'mustang1')
-        submission = yaml.load(open(
+        submission = yaml.safe_load(open(
             os.path.join(os.path.dirname(__file__), 'sample_jobs', 'mustang-ssh-multinode.yaml'), 'r'))
         target_group = 'arbitrary-group-id'  # for unit tests only
         jobs_dict = split_multinode_yaml(submission, target_group)
@@ -645,7 +623,7 @@ class TestYamlMultinode(TestCaseWithFactory):
 
         try:
             device_config = device.load_configuration(job_ctx)  # raw dict
-        except (jinja2.TemplateError, yaml.YAMLError, IOError) as exc:
+        except (jinja2.TemplateError, yaml.YAMLError, OSError) as exc:
             # FIXME: report the exceptions as useful user messages
             self.fail("[%d] jinja2 error: %s" % (host_job.id, exc))
         if not device_config or not isinstance(device_config, dict):
@@ -683,7 +661,7 @@ class TestYamlMultinode(TestCaseWithFactory):
         Tag.objects.all().delete()
         self.factory.ensure_tag('tap'),
         self.factory.ensure_tag('virtio')
-        submission = yaml.load(open(
+        submission = yaml.safe_load(open(
             os.path.join(os.path.dirname(__file__), 'sample_jobs', 'kvm-multinode.yaml'), 'r'))
         roles_dict = submission['protocols'][MultinodeProtocol.name]['roles']
         roles_dict['client']['tags'] = ['tap']
@@ -701,8 +679,35 @@ class TestYamlMultinode(TestCaseWithFactory):
                 else:
                     self.fail('unexpected role')
 
+        # all devices available
+        user = self.factory.make_user()
+        device_type = self.factory.make_device_type()
+
+        self.factory.make_device(device_type, 'fakeqemu1',
+                                 tags=[self.factory.ensure_tag('tap')])
+        self.factory.make_device(device_type, 'fakeqemu2',
+                                 tags=[self.factory.ensure_tag('virtio')])
+        job_list = TestJob.from_yaml_and_user(yaml.dump(submission), user)
+        self.assertEqual(len(job_list), 2)
+        # not enough devices with correct tags
+        roles_dict['client']['count'] = 2
+        submission['protocols'][MultinodeProtocol.name]['roles'] = roles_dict
+        self.factory.make_device(device_type, 'fakeqemu3',
+                                 tags=[self.factory.ensure_tag('wrong-tag')])
+        self.assertRaises(
+            DevicesUnavailableException,
+            TestJob.from_yaml_and_user,
+            yaml.dump(submission),
+            user
+        )
+        # enough devices with correct tags
+        self.factory.make_device(device_type, 'fakeqemu4',
+                                 tags=[self.factory.ensure_tag('tap')])
+        job_list = TestJob.from_yaml_and_user(yaml.dump(submission), user)
+        self.assertEqual(len(job_list), 3)
+
     def test_multinode_lxc(self):
-        submission = yaml.load(open(
+        submission = yaml.safe_load(open(
             os.path.join(os.path.dirname(__file__), 'sample_jobs', 'lxc-multinode.yaml'), 'r'))
         target_group = 'arbitrary-group-id'  # for unit tests only
 
@@ -723,7 +728,7 @@ class TestYamlMultinode(TestCaseWithFactory):
                 self.fail('Unrecognised role: %s' % role)
 
     def test_multinode_hikey(self):
-        submission = yaml.load(open(
+        submission = yaml.safe_load(open(
             os.path.join(os.path.dirname(__file__), 'sample_jobs', 'hikey_multinode.yaml'), 'r'))
         target_group = 'arbitrary-group-id'  # for unit tests only
 
@@ -750,7 +755,7 @@ class TestYamlMultinode(TestCaseWithFactory):
                 self.fail('Unrecognised role: %s' % role)
 
     def test_multinode_nexus4(self):
-        submission = yaml.load(open(
+        submission = yaml.safe_load(open(
             os.path.join(os.path.dirname(__file__), 'sample_jobs', 'nexus4_multinode.yaml'), 'r'))
         target_group = 'arbitrary-group-id'  # for unit tests only
 
@@ -775,7 +780,7 @@ class TestYamlMultinode(TestCaseWithFactory):
         device_type = self.factory.make_device_type()
         Device.objects.filter(device_type=device_type).delete()
         Tag.objects.all().delete()
-        submission = yaml.load(open(
+        submission = yaml.safe_load(open(
             os.path.join(os.path.dirname(__file__), 'sample_jobs', 'kvm-multinode.yaml'), 'r'))
         # no devices defined for the specified type
         self.assertRaises(DevicesUnavailableException, _pipeline_protocols, submission, user, yaml_data=None)
@@ -802,7 +807,7 @@ class TestYamlMultinode(TestCaseWithFactory):
         self.assertEqual(len(job_object_list), 2)
         for job in job_object_list:
             self.assertEqual(list(job.sub_jobs_list), job_object_list)
-            check = yaml.load(job.definition)
+            check = yaml.safe_load(job.definition)
             if check['protocols']['lava-multinode']['role'] == 'client':
                 self.assertEqual(
                     check['protocols']['lava-multinode']['tags'],
@@ -818,7 +823,7 @@ class TestYamlMultinode(TestCaseWithFactory):
     def test_multinode_group(self):
         user = self.factory.make_user()
         device_type = self.factory.make_device_type()
-        submission = yaml.load(open(
+        submission = yaml.safe_load(open(
             os.path.join(os.path.dirname(__file__), 'sample_jobs', 'kvm-multinode.yaml'), 'r'))
         self.factory.make_device(device_type, 'fakeqemu1')
         tag_list = [
@@ -830,8 +835,8 @@ class TestYamlMultinode(TestCaseWithFactory):
         job_object_list = _pipeline_protocols(submission, user, yaml.dump(submission))
         for job in job_object_list:
             self.assertEqual(list(job.sub_jobs_list), job_object_list)
-        check_one = yaml.load(job_object_list[0].definition)
-        check_two = yaml.load(job_object_list[1].definition)
+        check_one = yaml.safe_load(job_object_list[0].definition)
+        check_two = yaml.safe_load(job_object_list[1].definition)
         self.assertEqual(
             job_object_list[0].target_group,
             job_object_list[1].target_group
@@ -860,7 +865,7 @@ class TestYamlMultinode(TestCaseWithFactory):
     def test_multinode_definition(self):
         user = self.factory.make_user()
         device_type = self.factory.make_device_type()
-        submission = yaml.load(open(
+        submission = yaml.safe_load(open(
             os.path.join(os.path.dirname(__file__), 'sample_jobs', 'kvm-multinode.yaml'), 'r'))
         self.factory.make_device(device_type, 'fakeqemu1')
         self.factory.make_device(device_type, 'fakeqemu2')
@@ -885,7 +890,7 @@ class TestYamlMultinode(TestCaseWithFactory):
     def test_invalid_multinode(self):  # pylint: disable=too-many-locals
         user = self.factory.make_user()
         device_type = self.factory.make_device_type()
-        submission = yaml.load(open(
+        submission = yaml.safe_load(open(
             os.path.join(os.path.dirname(__file__), 'sample_jobs', 'kvm-multinode.yaml'), 'r'))
 
         tag_list = [
@@ -912,10 +917,10 @@ class TestYamlMultinode(TestCaseWithFactory):
             "%d.%d" % (int(job_object_list[0].id), 0))
         # FIXME: dispatcher master needs to make this kind of test more accessible.
         for job in job_object_list:
-            definition = yaml.load(job.definition)
+            definition = yaml.safe_load(job.definition)
             self.assertNotEqual(definition['protocols']['lava-multinode']['sub_id'], '')
             job.actual_device = Device.objects.get(hostname='fakeqemu1')
-            job_def = yaml.load(job.definition)
+            job_def = yaml.safe_load(job.definition)
             job_ctx = job_def.get('context', {})
             parser = JobParser()
             device = None
@@ -925,7 +930,7 @@ class TestYamlMultinode(TestCaseWithFactory):
 
                 try:
                     device_config = device.load_configuration(job_ctx)  # raw dict
-                except (jinja2.TemplateError, yaml.YAMLError, IOError) as exc:
+                except (jinja2.TemplateError, yaml.YAMLError, OSError) as exc:
                     # FIXME: report the exceptions as useful user messages
                     self.fail("[%d] jinja2 error: %s" % (job.id, exc))
                 if not device_config or not isinstance(device_config, dict):
@@ -966,7 +971,7 @@ class TestYamlMultinode(TestCaseWithFactory):
         self.factory.make_device(device_type, 'fakeqemu2')
         self.factory.make_device(device_type, 'fakeqemu3')
         self.factory.make_device(device_type, 'fakeqemu4')
-        submission = yaml.load(open(
+        submission = yaml.safe_load(open(
             os.path.join(os.path.dirname(__file__), 'sample_jobs', 'kvm-multinode.yaml'), 'r'))
         role_list = submission['protocols'][MultinodeProtocol.name]['roles']
         for role in role_list:
@@ -979,7 +984,7 @@ class TestYamlMultinode(TestCaseWithFactory):
         device_type = self.factory.make_device_type()
         self.factory.make_device(device_type, 'fakeqemu1')
         self.factory.make_device(device_type, 'fakeqemu2')
-        client_submission = yaml.load(open(
+        client_submission = yaml.safe_load(open(
             os.path.join(os.path.dirname(__file__), 'sample_jobs', 'kvm-multinode-client.yaml'), 'r'))
         job_ctx = client_submission.get('context', {})
         device = Device.objects.get(hostname='fakeqemu1')
@@ -1003,7 +1008,7 @@ class TestYamlMultinode(TestCaseWithFactory):
             },
             meta_dict)
         # simulate dynamic connection
-        dynamic = yaml.load(open(
+        dynamic = yaml.load(open(  # nosec - not suitable for safe_load
             os.path.join(os.path.dirname(__file__), 'pipeline_refs', 'connection-description.yaml'), 'r'))
         meta_dict = _get_action_metadata(dynamic['job']['actions'])
         self.assertEqual(
@@ -1020,6 +1025,7 @@ class TestYamlMultinode(TestCaseWithFactory):
         )
 
     @unittest.skipIf(not os.path.exists(SYS_CLASS_KVM), "Cannot use --enable-kvm")
+    @unittest.skipIf(check_rpcinfo(), "rpcinfo returns non-zero for nfs")
     def test_multinode_mixed_deploy(self):
         user = self.factory.make_user()
         device_type = self.factory.make_device_type()
@@ -1028,12 +1034,12 @@ class TestYamlMultinode(TestCaseWithFactory):
         bbb_type = self.factory.make_device_type('beaglebone-black')
         self.factory.make_device(hostname='bbb-01', device_type=bbb_type)
         self.factory.make_device(hostname='bbb-02', device_type=bbb_type)
-        submission = yaml.load(open(
+        submission = yaml.safe_load(open(
             os.path.join(os.path.dirname(__file__), 'sample_jobs', 'bbb-qemu-multinode.yaml'), 'r'))
         job_object_list = _pipeline_protocols(submission, user, yaml.dump(submission))
 
         for job in job_object_list:
-            definition = yaml.load(job.definition)
+            definition = yaml.safe_load(job.definition)
             self.assertNotEqual(definition['protocols']['lava-multinode']['sub_id'], '')
             sub_ids = [job.sub_id for job in job_object_list]
             self.assertEqual(len(set(sub_ids)), len(sub_ids))
@@ -1043,7 +1049,7 @@ class TestYamlMultinode(TestCaseWithFactory):
                 job.actual_device = Device.objects.get(hostname='bbb-01')
             else:
                 self.fail('Unrecognised device type: %s' % job.requested_device_type)
-            job_def = yaml.load(job.definition)
+            job_def = yaml.safe_load(job.definition)
             job_ctx = job_def.get('context', {})
             parser = JobParser()
             device_object = None
@@ -1052,7 +1058,7 @@ class TestYamlMultinode(TestCaseWithFactory):
 
                 try:
                     device_config = device.load_configuration(job_ctx)  # raw dict
-                except (jinja2.TemplateError, yaml.YAMLError, IOError) as exc:
+                except (jinja2.TemplateError, yaml.YAMLError, OSError) as exc:
                     # FIXME: report the exceptions as useful user messages
                     self.fail("[%d] jinja2 error: %s" % (job.id, exc))
                 if not device_config or not isinstance(device_config, dict):
@@ -1089,14 +1095,14 @@ class TestYamlMultinode(TestCaseWithFactory):
         bbb_type = self.factory.make_device_type('beaglebone-black')
         self.factory.make_device(hostname='bbb-01', device_type=bbb_type)
         self.factory.make_device(hostname='bbb-02', device_type=bbb_type)
-        submission = yaml.load(open(
+        submission = yaml.safe_load(open(
             os.path.join(os.path.dirname(__file__), 'sample_jobs', 'bbb-qemu-multinode.yaml'), 'r'))
         self.assertIn('protocols', submission)
         self.assertIn(MultinodeProtocol.name, submission['protocols'])
         submission['protocols'][MultinodeProtocol.name]['roles']['server']['essential'] = True
         job_object_list = _pipeline_protocols(submission, user, yaml.dump(submission))
         for job in job_object_list:
-            definition = yaml.load(job.definition)
+            definition = yaml.safe_load(job.definition)
             role = definition['protocols'][MultinodeProtocol.name]['role']
             self.assertNotEqual(definition['protocols']['lava-multinode']['sub_id'], '')
             if role == 'client':
@@ -1110,7 +1116,7 @@ class TestYamlMultinode(TestCaseWithFactory):
 class VlanInterfaces(TestCaseWithFactory):
 
     def setUp(self):
-        super(VlanInterfaces, self).setUp()
+        super().setUp()
         # YAML, pipeline only
         user = User.objects.create_user('test', 'e@mail.invalid', 'test')
         user.user_permissions.add(
@@ -1123,7 +1129,7 @@ class VlanInterfaces(TestCaseWithFactory):
         self.filename = os.path.join(os.path.dirname(__file__), 'sample_jobs', 'bbb-cubie-vlan-group.yaml')
 
     def test_vlan_interface(self):  # pylint: disable=too-many-locals
-        submission = yaml.load(open(self.filename, 'r'))
+        submission = yaml.safe_load(open(self.filename, 'r'))
         self.assertIn('protocols', submission)
         self.assertIn('lava-vland', submission['protocols'])
         roles = [role for role, _ in submission['protocols']['lava-vland'].items()]
