@@ -1,24 +1,25 @@
-# Copyright (C) 2010, 2011 Linaro Limited
+# -*- coding: utf-8 -*-
+# Copyright (C) 2010-2018 Linaro Limited
 #
 # Author: Zygmunt Krynicki <zygmunt.krynicki@linaro.org>
-# Author: Michael Hudson-Doyle <michael.hudson@linaro.org>
+#         Michael Hudson-Doyle <michael.hudson@linaro.org>
 #
-# This file is part of LAVA Server.
+# This file is part of LAVA.
 #
-# LAVA Server is free software: you can redistribute it and/or modify
+# LAVA is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License version 3
 # as published by the Free Software Foundation
 #
-# LAVA Server is distributed in the hope that it will be useful,
+# LAVA is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with LAVA Server.  If not, see <http://www.gnu.org/licenses/>.
+# along with LAVA.  If not, see <http://www.gnu.org/licenses/>.
 
 import os
-import subprocess
+import subprocess  # nosec internal
 import xmlrpc.client
 import yaml
 
@@ -79,7 +80,7 @@ class LavaSystemAPI(SystemAPI):
 
         changelog = '/usr/share/doc/lava-server/changelog.Debian.gz'
         if os.path.exists(changelog):
-            deb_version = subprocess.check_output((
+            deb_version = subprocess.check_output((  # nosec internal
                 'dpkg-query', '-W', "-f=${Version}\n",
                 "lava-server")).strip().decode('utf-8')
             return deb_version
@@ -167,7 +168,7 @@ class LavaSystemAPI(SystemAPI):
         if os.path.exists(filename):
             try:
                 with open(filename, 'r') as output:
-                    master = yaml.load(output)
+                    master = yaml.safe_load(output)
             except yaml.YAMLError as exc:
                 return master
         if master:
@@ -178,7 +179,7 @@ class LavaSystemAPI(SystemAPI):
         if os.path.exists(filename):
             try:
                 with open(filename, 'r') as output:
-                    log_config = yaml.load(output)
+                    log_config = yaml.safe_load(output)
             except yaml.YAMLError as exc:
                 return log_config
         if log_config:
